@@ -29,6 +29,7 @@
 #include "gdaui-basic-form.h"
 #include "gdaui-data-filter.h"
 #include "internal/utility.h"
+#include "data-entries/gdaui-entry-shell.h"
 
 static void gdaui_raw_form_class_init (GdauiRawFormClass * class);
 static void gdaui_raw_form_init (GdauiRawForm *wid);
@@ -43,8 +44,6 @@ static void gdaui_raw_form_get_property (GObject *object,
 					 GValue *value,
 					 GParamSpec *pspec);
 
-static void gdaui_raw_form_initialize (GdauiRawForm *form, GtkWidget *layout, GHashTable *box_widgets);
-
 static GError *iter_validate_set_cb (GdaDataModelIter *iter, GdauiRawForm *form);
 static void iter_row_changed_cb (GdaDataModelIter *iter, gint row, GdauiRawForm *form);
 static void proxy_changed_cb (GdaDataProxy *proxy, GdauiRawForm *form);
@@ -158,18 +157,27 @@ gdaui_raw_form_selector_init (GdauiDataSelectorIface *iface)
 }
 
 static void
-gdaui_raw_form_class_init (GdauiRawFormClass * class)
+basic_form_layout_changed (GdauiBasicForm *bform)
 {
-	GObjectClass   *object_class = G_OBJECT_CLASS (class);
+	GdauiRawForm *form = GDAUI_RAW_FORM (bform);
+	iter_row_changed_cb (form->priv->iter, gda_data_model_iter_get_row (form->priv->iter), form);
+}
+
+static void
+gdaui_raw_form_class_init (GdauiRawFormClass *class)
+{
+	GObjectClass *object_class = G_OBJECT_CLASS (class);
 
 	parent_class = g_type_class_peek_parent (class);
 	object_class->dispose = gdaui_raw_form_dispose;
+	GDAUI_BASIC_FORM_CLASS (class)->layout_changed = basic_form_layout_changed;
 
 	/* Properties */
         object_class->set_property = gdaui_raw_form_set_property;
         object_class->get_property = gdaui_raw_form_get_property;
 	g_object_class_install_property (object_class, PROP_MODEL,
-					 g_param_spec_object ("model", _("Data to display"), NULL, GDA_TYPE_DATA_MODEL,
+					 g_param_spec_object ("model", _("Data to display"),
+							      NULL, GDA_TYPE_DATA_MODEL,
 							      G_PARAM_READABLE | G_PARAM_WRITABLE));
 }
 
@@ -355,6 +363,21 @@ gdaui_raw_form_dispose (GObject *object)
 	parent_class->dispose (object);
 }
 
+static gboolean
+proxy_reset_was_soft (GdauiRawForm *form, GdaDataModel *new_model)
+{
+	GdaDataModelIter *iter;
+	gboolean retval = FALSE;
+
+	if (!new_model || (new_model != (GdaDataModel*) form->priv->proxy))
+		return FALSE;
+
+	iter = gda_data_model_create_iter (new_model);
+	retval = ! _gdaui_utility_iter_differ (form->priv->iter, iter);
+	g_object_unref (iter);
+	return retval;
+}
+
 static void
 gdaui_raw_form_set_property (GObject *object,
 			     guint param_id,
@@ -367,69 +390,97 @@ gdaui_raw_form_set_property (GObject *object,
         form = GDAUI_RAW_FORM (object);
         if (form->priv) {
                 switch (param_id) {
-		case PROP_MODEL:
-			ptr = GDA_DATA_MODEL(g_value_get_object (value));
+		case PROP_MODEL: {
+			gboolean reset;
+
+			ptr = GDA_DATA_MODEL (g_value_get_object (value));
 			if (ptr)
 				g_return_if_fail (GDA_IS_DATA_MODEL (ptr));
-			if (form->priv->proxy) {
-				/* remove old data model settings */
-				g_signal_handlers_disconnect_by_func (form->priv->iter,
-								      G_CALLBACK (iter_row_changed_cb), form);
-				g_signal_handlers_disconnect_by_func (form->priv->iter,
-								      G_CALLBACK (iter_validate_set_cb), form);
+			reset = !proxy_reset_was_soft (form, (GdaDataModel*) ptr);
 
-				g_object_unref (G_OBJECT (form->priv->iter));
-				form->priv->iter = NULL;
+			if (reset) {
+				if (form->priv->proxy) {
+					/* remove old data model settings */
+					g_signal_handlers_disconnect_by_func (form->priv->iter,
+									      G_CALLBACK (iter_row_changed_cb),
+									      form);
+					g_signal_handlers_disconnect_by_func (form->priv->iter,
+									      G_CALLBACK (iter_validate_set_cb),
+									      form);
+					g_object_unref (G_OBJECT (form->priv->iter));
+					form->priv->iter = NULL;
 
-				g_signal_handlers_disconnect_by_func (G_OBJECT (form->priv->proxy),
-								      G_CALLBACK (proxy_row_inserted_or_removed_cb),
-								      form);
-				g_signal_handlers_disconnect_by_func (G_OBJECT (form->priv->proxy),
-								      G_CALLBACK (proxy_changed_cb), form);
-				g_signal_handlers_disconnect_by_func (G_OBJECT (form->priv->proxy),
-								      G_CALLBACK (proxy_reset_cb), form);
-				g_object_unref (G_OBJECT (form->priv->proxy));
-				form->priv->proxy = NULL;
-				form->priv->model = NULL;
-			}
-			if (ptr) {
-				/* handle the data model */
-				if (GDA_IS_DATA_PROXY (ptr)) {
-					form->priv->proxy = (GdaDataProxy *) ptr;
-					g_object_ref (ptr);
+					g_signal_handlers_disconnect_by_func (G_OBJECT (form->priv->proxy),
+									      G_CALLBACK (proxy_row_inserted_or_removed_cb),
+									      form);
+					g_signal_handlers_disconnect_by_func (G_OBJECT (form->priv->proxy),
+									      G_CALLBACK (proxy_changed_cb), form);
+					g_signal_handlers_disconnect_by_func (G_OBJECT (form->priv->proxy),
+									      G_CALLBACK (proxy_reset_cb), form);
+					g_object_unref (G_OBJECT (form->priv->proxy));
+					form->priv->proxy = NULL;
+					form->priv->model = NULL;
 				}
-				else
-					form->priv->proxy = (GdaDataProxy *) gda_data_proxy_new ((GdaDataModel*) ptr);
+				if (ptr) {
+					/* handle the data model */
+					if (GDA_IS_DATA_PROXY (ptr)) {
+						form->priv->proxy = (GdaDataProxy *) ptr;
+						g_object_ref (ptr);
+					}
+					else
+						form->priv->proxy = (GdaDataProxy *) gda_data_proxy_new ((GdaDataModel*) ptr);
 
-				form->priv->model = gda_data_proxy_get_proxied_model (form->priv->proxy);
+					form->priv->model = gda_data_proxy_get_proxied_model (form->priv->proxy);
+					form->priv->iter = gda_data_model_create_iter ((GdaDataModel *) form->priv->proxy);
+					gda_data_model_iter_move_to_row (form->priv->iter, 0);
+					
+					g_signal_connect (form->priv->iter, "validate-set",
+							  G_CALLBACK (iter_validate_set_cb), form);
+					g_signal_connect (form->priv->iter, "row-changed",
+							  G_CALLBACK (iter_row_changed_cb), form);
+					
+					g_signal_connect (G_OBJECT (form->priv->proxy), "row_inserted",
+							  G_CALLBACK (proxy_row_inserted_or_removed_cb), form);
+					g_signal_connect (G_OBJECT (form->priv->proxy), "row_removed",
+							  G_CALLBACK (proxy_row_inserted_or_removed_cb), form);
+					g_signal_connect (G_OBJECT (form->priv->proxy), "changed",
+							  G_CALLBACK (proxy_changed_cb), form);
+					g_signal_connect (G_OBJECT (form->priv->proxy), "reset",
+							  G_CALLBACK (proxy_reset_cb), form);
 
-				form->priv->iter = gda_data_model_create_iter ((GdaDataModel *) form->priv->proxy);
-				gda_data_model_iter_move_to_row (form->priv->iter, 0);
+					/* we don't want chuncking */
+					g_object_set (object, "paramlist", form->priv->iter, NULL);
+					gda_data_proxy_set_sample_size (form->priv->proxy, 0);
 
-				g_signal_connect (form->priv->iter, "validate-set",
-						  G_CALLBACK (iter_validate_set_cb), form);
-				g_signal_connect (form->priv->iter, "row-changed",
-						  G_CALLBACK (iter_row_changed_cb), form);
+					/* handle invalie iterators' GdaHolder */
+					GSList *list;
+					for (list = GDA_SET (form->priv->iter)->holders; list; list = list->next) {
+						GtkWidget *entry;
+						entry = gdaui_basic_form_get_entry_widget (GDAUI_BASIC_FORM (form),
+											   (GdaHolder*) list->data);
+						if (entry)
+							gdaui_entry_shell_set_unknown ((GdauiEntryShell*) entry,
+										       !gda_holder_is_valid ((GdaHolder*) list->data));
+					}
 
-				g_signal_connect (G_OBJECT (form->priv->proxy), "row_inserted",
-						  G_CALLBACK (proxy_row_inserted_or_removed_cb), form);
-				g_signal_connect (G_OBJECT (form->priv->proxy), "row_removed",
-						  G_CALLBACK (proxy_row_inserted_or_removed_cb), form);
-				g_signal_connect (G_OBJECT (form->priv->proxy), "changed",
-						  G_CALLBACK (proxy_changed_cb), form);
-				g_signal_connect (G_OBJECT (form->priv->proxy), "reset",
-						  G_CALLBACK (proxy_reset_cb), form);
-
-				/* we don't want chuncking */
-				g_object_set (object, "paramlist", form->priv->iter, NULL);
-				gda_data_proxy_set_sample_size (form->priv->proxy, 0);
-				gdaui_raw_form_initialize (form, NULL, NULL);
-				iter_row_changed_cb (form->priv->iter, gda_data_model_iter_get_row (form->priv->iter), form);
+					/* actions */
+					if (gda_data_proxy_is_read_only (form->priv->proxy))
+						g_object_set ((GObject*) form, "show-actions", FALSE, NULL);
+					
+					/* data display update */
+					proxy_changed_cb (form->priv->proxy, form);					
+				}
+				if (form->priv->iter)
+					iter_row_changed_cb (form->priv->iter,
+							     gda_data_model_iter_get_row (form->priv->iter), form);
+				gdaui_raw_form_widget_set_write_mode ((GdauiDataProxy *) form,
+								      form->priv->write_mode);
 			}
-
-			gdaui_raw_form_widget_set_write_mode ((GdauiDataProxy *) form, form->priv->write_mode);
+			else
+				gda_data_model_iter_move_to_row (form->priv->iter, 0);
 			g_signal_emit_by_name (object, "proxy-changed", form->priv->proxy);
 			break;
+		}
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 			break;
@@ -528,49 +579,6 @@ proxy_row_inserted_or_removed_cb (GdaDataProxy *proxy, gint row, GdauiRawForm *f
 			gda_data_model_iter_move_to_row (form->priv->iter, row > 0 ? row - 1 : 0);
 }
 
-/*
- * Real initialization
- */
-static void
-gdaui_raw_form_initialize (GdauiRawForm *form, GtkWidget *layout, GHashTable *box_widgets)
-{
-	/*
-	 * handling of the layout
-	 */
-	if (layout && box_widgets) {
-		/* REM: the @box_widgets has keys for query fields, and we need to provide
-		   gdaui_basic_form_new_in_layout() with a hash table which has keys for GdaSetNode;
-		   this is why we need to use another hash table: 'fbw' */
-
-		TO_IMPLEMENT;
-		/* GSList *list; */
-		/* 		GdaHolder *param; */
-		/* 		GdaSetNode *node; */
-		/* 		gpointer widget; */
-
-		/* 		fbw = g_hash_table_new (NULL, NULL); */
-		/* 		g_object_get (G_OBJECT (orig_query), "really_all_fields", &list, NULL); */
-		/* 		while (list) { */
-		/* 			widget = g_hash_table_lookup (box_widgets, list->data); */
-		/* 			if (widget) { */
-		/* 				node = gdaui_work_core_find_context_node (form->priv->core, GDA_QUERY_FIELD (list->data)); */
-		/* 				if (node) */
-		/* 					g_hash_table_insert (fbw, node, widget); */
-		/* 			} */
-		/* 			list = g_slist_next (list); */
-		/* 		} */
-	}
-
-	/*
-	 * the form itself
-	 */
-	if (!layout && gda_data_proxy_is_read_only (form->priv->proxy))
-		g_object_set ((GObject*) form, "show-actions", FALSE, NULL);
-
-	/* data display update */
-	proxy_changed_cb (form->priv->proxy, form);
-}
-
 /*
  *
  * Modification buttons (Commit changes, Reset form, New entry, Delete)
@@ -583,10 +591,12 @@ action_new_cb (GtkAction *action, GdauiRawForm *form)
 	GError *error = NULL;
 	GSList *list;
 
-	if (form->priv->write_mode >= GDAUI_DATA_PROXY_WRITE_ON_ROW_CHANGE)
+	if (form->priv->write_mode >= GDAUI_DATA_PROXY_WRITE_ON_ROW_CHANGE) {
+		/* validate current row (forces calling iter_validate_set_cb()) */
 		if (gda_data_model_iter_is_valid (form->priv->iter) &&
 		    ! gda_set_is_valid (GDA_SET (form->priv->iter), NULL))
 			return;
+	}
 
 	/* append a row in the proxy */
 	g_signal_handlers_block_by_func (form, G_CALLBACK (form_holder_changed_cb), NULL);
@@ -618,7 +628,13 @@ action_new_cb (GtkAction *action, GdauiRawForm *form)
 		}
 		else
 			g_object_unref (param);
+
+		GtkWidget *entry;
+		entry = gdaui_basic_form_get_entry_widget (GDAUI_BASIC_FORM (form), (GdaHolder*) list->data);
+		if (entry)
+			gdaui_entry_shell_set_unknown ((GdauiEntryShell*) entry, FALSE);
 	}
+
 	g_signal_handlers_unblock_by_func (form, G_CALLBACK (form_holder_changed_cb), NULL);
 	form_holder_changed_cb (form, NULL);
 }
@@ -637,7 +653,7 @@ action_delete_cb (GtkAction *action, GdauiRawForm *form)
 		gint newrow;
 
 		newrow = gda_data_model_iter_get_row (form->priv->iter);
-		if (row == newrow) {/* => row has been marked as delete but nit yet really deleted */
+		if (row == newrow) {/* => row has been marked as delete but not yet really deleted */
 			GError *error = NULL;
 			if (!gda_data_proxy_apply_row_changes (form->priv->proxy, row, &error)) {
 				_gdaui_utility_display_error ((GdauiDataProxy *) form, TRUE, error);
@@ -694,7 +710,8 @@ action_commit_cb (GtkAction *action, GdauiRawForm *form)
 		g_error_free (error);
 	}
 
-	iter_row_changed_cb (form->priv->iter, gda_data_model_iter_get_row (form->priv->iter), form);
+	/* get to a correct selected row */
+	for (; !gda_data_model_iter_move_to_row (form->priv->iter, row) && (row >= 0); row--);
 }
 
 static void
