@@ -105,6 +105,13 @@ gda_batch_class_init (GdaBatchClass * klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	parent_class = g_type_class_peek_parent (klass);
 
+	/**
+	 * GdaBatch::changed
+	 * @batch: the #GdaBatch object
+	 * @changed_stmt: the statement which has been changed
+	 *
+	 * Gets emitted whenever a #GdaStatement in the @batch object changes
+	 */
 	gda_batch_signals[CHANGED] =
 		g_signal_new ("changed",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -430,7 +437,7 @@ gda_batch_get_parameters (GdaBatch *batch, GdaSet **out_params, GError **error)
 						if (!eholder ||
 						    (gda_holder_get_g_type (eholder) != (gda_holder_get_g_type (holder)))) {
 							/* error */
-							g_set_error (error, GDA_BATCH_ERROR, 0,
+							g_set_error (error, GDA_BATCH_ERROR, GDA_BATCH_CONFLICTING_PARAMETER_ERROR,
 								     _("Conflicting parameter '%s'"), gda_holder_get_id (holder));
 							g_object_unref (tmpset);
 							g_object_unref (set);
