@@ -86,7 +86,7 @@ GQuark gda_sql_builder_error_quark (void) {
 GType
 gda_sql_builder_get_type (void) {
 	static GType type = 0;
-	
+
 	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GdaSqlBuilderClass),
@@ -99,7 +99,7 @@ gda_sql_builder_get_type (void) {
 			0,
 			(GInstanceInitFunc) gda_sql_builder_init
 		};
-		
+
 		g_static_rec_mutex_lock (&init_mutex);
 		if (type == 0)
 			type = g_type_register_static (G_TYPE_OBJECT, "GdaSqlBuilder", &info, 0);
@@ -110,12 +110,12 @@ gda_sql_builder_get_type (void) {
 
 
 static void
-gda_sql_builder_class_init (GdaSqlBuilderClass *klass) 
+gda_sql_builder_class_init (GdaSqlBuilderClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
-	
+
 	/* Properties */
 	object_class->set_property = gda_sql_builder_set_property;
 	object_class->get_property = gda_sql_builder_get_property;
@@ -151,7 +151,7 @@ any_part_free (SqlPart *part)
 }
 
 static void
-gda_sql_builder_init (GdaSqlBuilder *builder) 
+gda_sql_builder_init (GdaSqlBuilder *builder)
 {
 	builder->priv = g_new0 (GdaSqlBuilderPrivate, 1);
 	builder->priv->main_stmt = NULL;
@@ -174,7 +174,7 @@ gda_sql_builder_init (GdaSqlBuilder *builder)
  * Since: 4.2
  */
 GdaSqlBuilder *
-gda_sql_builder_new (GdaSqlStatementType stmt_type) 
+gda_sql_builder_new (GdaSqlStatementType stmt_type)
 {
 	GdaSqlBuilder *builder;
 
@@ -184,12 +184,12 @@ gda_sql_builder_new (GdaSqlStatementType stmt_type)
 
 
 static void
-gda_sql_builder_dispose (GObject *object) 
+gda_sql_builder_dispose (GObject *object)
 {
 	GdaSqlBuilder *builder;
-	
+
 	g_return_if_fail (GDA_IS_SQL_BUILDER (object));
-	
+
 	builder = GDA_SQL_BUILDER (object);
 	if (builder->priv) {
 		if (builder->priv->main_stmt) {
@@ -201,7 +201,7 @@ gda_sql_builder_dispose (GObject *object)
 			builder->priv->parts_hash = NULL;
 		}
 	}
-	
+
 	/* parent class */
 	parent_class->dispose (object);
 }
@@ -210,16 +210,16 @@ static void
 gda_sql_builder_finalize (GObject *object)
 {
 	GdaSqlBuilder *builder;
-	
+
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (GDA_IS_SQL_BUILDER (object));
-	
+
 	builder = GDA_SQL_BUILDER (object);
 	if (builder->priv) {
 		g_free (builder->priv);
 		builder->priv = NULL;
 	}
-	
+
 	/* parent class */
 	parent_class->finalize (object);
 }
@@ -228,11 +228,11 @@ static void
 gda_sql_builder_set_property (GObject *object,
 			     guint param_id,
 			     const GValue *value,
-			     GParamSpec *pspec) 
+			     GParamSpec *pspec)
 {
 	GdaSqlBuilder *builder;
 	GdaSqlStatementType stmt_type;
-  
+
 	builder = GDA_SQL_BUILDER (object);
 	if (builder->priv) {
 		switch (param_id) {
@@ -258,11 +258,11 @@ static void
 gda_sql_builder_get_property (GObject *object,
 			     guint param_id,
 			     GValue *value,
-			     GParamSpec *pspec) 
+			     GParamSpec *pspec)
 {
 	GdaSqlBuilder *builder;
 	builder = GDA_SQL_BUILDER (object);
-	
+
 	if (builder->priv) {
 		switch (param_id) {
 		}
@@ -371,7 +371,7 @@ GdaSqlStatement *
 gda_sql_builder_get_sql_statement (GdaSqlBuilder *builder, gboolean copy_it)
 {
 	g_return_val_if_fail (GDA_IS_SQL_BUILDER (builder), NULL);
-	if (!builder->priv->main_stmt) 
+	if (!builder->priv->main_stmt)
 		return NULL;
 	if (copy_it)
 		return gda_sql_statement_copy (builder->priv->main_stmt);
@@ -498,35 +498,40 @@ gda_sql_builder_set_where (GdaSqlBuilder *builder, guint cond_id)
  *
  * Add a selected selected item to the SELECT statement.
  *
+ * Returns: the ID of the added field, or 0 if there was an error
+ *
  * Since: 4.2
  */
-void
+guint
 gda_sql_builder_select_add_field (GdaSqlBuilder *builder, const gchar *field_name, const gchar *table_name, const gchar *alias)
 {
 	gchar *tmp;
-	g_return_if_fail (GDA_IS_SQL_BUILDER (builder));
-	g_return_if_fail (builder->priv->main_stmt);
+	g_return_val_if_fail (GDA_IS_SQL_BUILDER (builder), 0);
+	g_return_val_if_fail (builder->priv->main_stmt, 0);
 	if (builder->priv->main_stmt->stmt_type != GDA_SQL_STATEMENT_SELECT) {
 		g_warning (_("Wrong statement type"));
-		return;
+		return 0;
 	}
-	g_return_if_fail (field_name && *field_name);
+	g_return_val_if_fail (field_name && *field_name, 0);
 
 	if (table_name && *table_name)
 		tmp = g_strdup_printf ("%s.%s", table_name, field_name);
 	else
 		tmp = (gchar*) field_name;
 
+	const guint field_id = gda_sql_builder_add_id (builder, 0, tmp);
 	if (alias && *alias)
 		gda_sql_builder_add_field_id (builder,
-					      gda_sql_builder_add_id (builder, 0, tmp),
+					      field_id,
 					      gda_sql_builder_add_id (builder, 0, alias));
 	else
 		gda_sql_builder_add_field_id (builder,
-					      gda_sql_builder_add_id (builder, 0, tmp),
+					      field_id,
 					      0);
 	if (table_name)
 		g_free (tmp);
+
+    return field_id;
 }
 
 static GValue *
@@ -553,7 +558,7 @@ create_typed_value (GType type, va_list *ap)
  *
  * Valid only for: INSERT, UPDATE statements.
  *
- * Specifies that the field represented by @field_name will be set to the value identified 
+ * Specifies that the field represented by @field_name will be set to the value identified
  * by @... of type @type.
  *
  * Since: 4.2
@@ -591,11 +596,11 @@ gda_sql_builder_add_field (GdaSqlBuilder *builder, const gchar *field_name, GTyp
  * gda_sql_builder_add_field_value
  * @builder: a #GdaSqlBuilder object
  * @field_name: a field name
- * @value: value to set the field to
+ * @value: value to set the field to, or %NULL or a GDA_TYPE_NULL value to represent an SQL NULL
  *
  * Valid only for: INSERT, UPDATE statements.
  *
- * Specifies that the field represented by @field_name will be set to the value identified 
+ * Specifies that the field represented by @field_name will be set to the value identified
  * by @value
  *
  * Since: 4.2
@@ -606,7 +611,6 @@ gda_sql_builder_add_field_value (GdaSqlBuilder *builder, const gchar *field_name
 	g_return_if_fail (GDA_IS_SQL_BUILDER (builder));
 	g_return_if_fail (builder->priv->main_stmt);
 	g_return_if_fail (field_name && *field_name);
-	g_return_if_fail (value);
 
 	if ((builder->priv->main_stmt->stmt_type != GDA_SQL_STATEMENT_UPDATE) &&
 	    (builder->priv->main_stmt->stmt_type != GDA_SQL_STATEMENT_INSERT)) {
@@ -628,7 +632,7 @@ gda_sql_builder_add_field_value (GdaSqlBuilder *builder, const gchar *field_name
  *
  * Valid only for: INSERT, UPDATE, SELECT statements
  * <itemizedlist>
- * <listitem><para>For UPDATE: specifies that the field represented by @field_id will be set to the value identified 
+ * <listitem><para>For UPDATE: specifies that the field represented by @field_id will be set to the value identified
  *    by @value_id.</para></listitem>
  * <listitem><para>For SELECT: add a selected item to the statement, and if @value_id is not %0, then use it as an
  *    alias</para></listitem>
@@ -685,7 +689,7 @@ gda_sql_builder_add_field_id (GdaSqlBuilder *builder, guint field_id, guint valu
 	}
 	case GDA_SQL_STATEMENT_INSERT:{
 		GdaSqlStatementInsert *ins = (GdaSqlStatementInsert*) builder->priv->main_stmt->contents;
-		
+
 		if (field_expr->select) {
 			switch (GDA_SQL_ANY_PART (field_expr->select)->type) {
 			case GDA_SQL_STATEMENT_SELECT:
@@ -701,7 +705,7 @@ gda_sql_builder_add_field_id (GdaSqlBuilder *builder, guint field_id, guint valu
 		else {
 			GdaSqlField *field = gda_sql_field_new (GDA_SQL_ANY_PART (ins));
 			field->field_name = g_value_dup_string (field_expr->value);
-			
+
 			ins->fields_list = g_slist_append (ins->fields_list, field);
 			if (value_part) {
 				if (! ins->values_list)
@@ -709,7 +713,7 @@ gda_sql_builder_add_field_id (GdaSqlBuilder *builder, guint field_id, guint valu
 									   g_slist_append (NULL,
 									   use_part (value_part, GDA_SQL_ANY_PART (ins))));
 				else
-					ins->values_list->data = g_slist_append ((GSList*) ins->values_list->data, 
+					ins->values_list->data = g_slist_append ((GSList*) ins->values_list->data,
 										 use_part (value_part, GDA_SQL_ANY_PART (ins)));
 			}
 		}
@@ -739,7 +743,7 @@ gda_sql_builder_add_field_id (GdaSqlBuilder *builder, guint field_id, guint valu
  * @builder: a #GdaSqlBuilder object
  * @id: the requested ID, or 0 if to be determined by @builder
  * @dh: a #GdaDataHandler to use, or %NULL
- * @value: value to set the expression to
+ * @value: value to set the expression to, or %NULL or a GDA_TYPE_NULL value to represent an SQL NULL
  *
  * Defines an expression in @builder which may be reused to build other parts of a statement.
  *
@@ -755,25 +759,27 @@ gda_sql_builder_add_expr_value (GdaSqlBuilder *builder, guint id, GdaDataHandler
 {
 	g_return_val_if_fail (GDA_IS_SQL_BUILDER (builder), 0);
 	g_return_val_if_fail (builder->priv->main_stmt, 0);
-	g_return_val_if_fail (value != NULL, 0);
 
-	GType type = G_VALUE_TYPE(value);
 	gchar *str;
 
-	if (!dh)
-		dh = gda_get_default_handler (type);
-	else {
-		if (! gda_data_handler_accepts_g_type (dh, type)) {
+	if (value && (G_VALUE_TYPE (value) != GDA_TYPE_NULL)) {
+		GType type = G_VALUE_TYPE (value);
+		if (!dh)
+			dh = gda_get_default_handler (type);
+		else {
+			if (! gda_data_handler_accepts_g_type (dh, type)) {
+				g_warning (_("Unhandled data type '%s'"), g_type_name (type));
+				return 0;
+			}
+		}
+		if (!dh) {
 			g_warning (_("Unhandled data type '%s'"), g_type_name (type));
 			return 0;
 		}
+		str = gda_data_handler_get_sql_from_value (dh, value);
 	}
-	if (!dh) {
-		g_warning (_("Unhandled data type '%s'"), g_type_name (type));
-		return 0;
-	}
-
-	str = gda_data_handler_get_sql_from_value (dh, value);
+	else
+		str = g_strdup ("NULL");
 
 	if (str) {
 		GdaSqlExpr *expr;
@@ -783,7 +789,8 @@ gda_sql_builder_add_expr_value (GdaSqlBuilder *builder, guint id, GdaDataHandler
 		return add_part (builder, id, (GdaSqlAnyPart *) expr);
 	}
 	else {
-		g_warning (_("Could not convert value to type '%s'"), g_type_name (type));
+		g_warning (_("Could not convert value to type '%s'"),
+			   g_type_name (G_VALUE_TYPE (value)));
 		return 0;
 	}
 }
@@ -822,7 +829,7 @@ gda_sql_builder_add_expr (GdaSqlBuilder *builder, guint id, GdaDataHandler *dh,
 {
 	g_return_val_if_fail (GDA_IS_SQL_BUILDER (builder), 0);
 	g_return_val_if_fail (builder->priv->main_stmt, 0);
-	
+
 	va_list ap;
 	GValue *value;
 	guint retval;
@@ -873,7 +880,7 @@ gda_sql_builder_add_id (GdaSqlBuilder *builder, guint id, const gchar *string)
 {
 	g_return_val_if_fail (GDA_IS_SQL_BUILDER (builder), 0);
 	g_return_val_if_fail (builder->priv->main_stmt, 0);
-	
+
 	GdaSqlExpr *expr;
 	expr = gda_sql_expr_new (NULL);
 	if (string) {
@@ -881,7 +888,7 @@ gda_sql_builder_add_id (GdaSqlBuilder *builder, guint id, const gchar *string)
 		g_value_set_string (expr->value, string);
 		expr->value_is_ident = (gpointer) 0x1;
 	}
-	
+
 	return add_part (builder, id, (GdaSqlAnyPart *) expr);
 }
 
@@ -917,7 +924,7 @@ gda_sql_builder_add_param (GdaSqlBuilder *builder, guint id, const gchar *param_
 	g_return_val_if_fail (GDA_IS_SQL_BUILDER (builder), 0);
 	g_return_val_if_fail (builder->priv->main_stmt, 0);
 	g_return_val_if_fail (param_name && *param_name, 0);
-	
+
 	GdaSqlExpr *expr;
 	expr = gda_sql_expr_new (NULL);
 	expr->param_spec = g_new0 (GdaSqlParamSpec, 1);
@@ -955,7 +962,7 @@ gda_sql_builder_add_cond (GdaSqlBuilder *builder, guint id, GdaSqlOperatorType o
 	if (!p1)
 		return 0;
 	p2 = get_part (builder, op2, GDA_SQL_ANY_EXPR);
-	
+
 	GdaSqlExpr *expr;
 	expr = gda_sql_expr_new (NULL);
 	expr->cond = gda_sql_operation_new (GDA_SQL_ANY_PART (expr));
@@ -968,7 +975,7 @@ gda_sql_builder_add_cond (GdaSqlBuilder *builder, guint id, GdaSqlOperatorType o
 		p3 = get_part (builder, op3, GDA_SQL_ANY_EXPR);
 		if (p3)
 			expr->cond->operands = g_slist_append (expr->cond->operands,
-							       use_part (p3, GDA_SQL_ANY_PART (expr->cond)));	
+							       use_part (p3, GDA_SQL_ANY_PART (expr->cond)));
 	}
 
 	return add_part (builder, id, (GdaSqlAnyPart *) expr);
@@ -1021,7 +1028,7 @@ gda_sql_builder_add_cond_v (GdaSqlBuilder *builder, guint id, GdaSqlOperatorType
 		else
 			return op_ids [0]; /* return the same ID as none was specified */
 	}
-	
+
 	GdaSqlExpr *expr;
 	expr = gda_sql_expr_new (NULL);
 	expr->cond = gda_sql_operation_new (GDA_SQL_ANY_PART (expr));
@@ -1060,7 +1067,7 @@ gda_sql_builder_select_add_target_id (GdaSqlBuilder *builder, guint id, guint ta
 {
 	g_return_val_if_fail (GDA_IS_SQL_BUILDER (builder), 0);
 	g_return_val_if_fail (builder->priv->main_stmt, 0);
-	
+
 	if (builder->priv->main_stmt->stmt_type != GDA_SQL_STATEMENT_SELECT) {
 		g_warning (_("Wrong statement type"));
 		return 0;
@@ -1070,7 +1077,7 @@ gda_sql_builder_select_add_target_id (GdaSqlBuilder *builder, guint id, guint ta
 	p = get_part (builder, table_id, GDA_SQL_ANY_EXPR);
 	if (!p)
 		return 0;
-	
+
 	BuildTarget *btarget;
 	GdaSqlStatementSelect *sel = (GdaSqlStatementSelect*) builder->priv->main_stmt->contents;
 	btarget = g_new0 (BuildTarget, 1);
@@ -1080,12 +1087,12 @@ gda_sql_builder_select_add_target_id (GdaSqlBuilder *builder, guint id, guint ta
 		btarget->part_id = id;
 	else
 		btarget->part_id = builder->priv->next_assigned_id --;
-	
+
 	((GdaSqlSelectTarget*) btarget)->expr = (GdaSqlExpr*) use_part (p, GDA_SQL_ANY_PART (btarget));
-	if (alias) 
+	if (alias && *alias)
 		((GdaSqlSelectTarget*) btarget)->as = g_strdup (alias);
 	if (g_value_get_string (((GdaSqlSelectTarget*) btarget)->expr->value))
-		((GdaSqlSelectTarget*) btarget)->table_name = 
+		((GdaSqlSelectTarget*) btarget)->table_name =
 			g_value_dup_string (((GdaSqlSelectTarget*) btarget)->expr->value);
 
 	/* add target to sel->from. NOTE: @btarget is NOT added to the "repository" or GdaSqlAnyPart parts
@@ -1124,7 +1131,7 @@ gda_sql_builder_select_add_target (GdaSqlBuilder *builder, const gchar *table_na
 	g_return_val_if_fail (table_name && *table_name, 0);
 
 	id = gda_sql_builder_select_add_target_id (builder,
-						   0, 
+						   0,
 						   gda_sql_builder_add_id (builder, 0, table_name),
 						   alias);
 	return id;
@@ -1159,7 +1166,7 @@ gda_sql_builder_select_join_targets (GdaSqlBuilder *builder, guint id,
 {
 	g_return_val_if_fail (GDA_IS_SQL_BUILDER (builder), 0);
 	g_return_val_if_fail (builder->priv->main_stmt, 0);
-	
+
 	if (builder->priv->main_stmt->stmt_type != GDA_SQL_STATEMENT_SELECT) {
 		g_warning (_("Wrong statement type"));
 		return 0;
@@ -1185,11 +1192,11 @@ gda_sql_builder_select_join_targets (GdaSqlBuilder *builder, guint id,
 		g_warning (_("Unknown part ID %u"), (left_pos == -1) ? left_pos : right_pos);
 		return 0;
 	}
-	
+
 	if (left_pos > right_pos) {
 		TO_IMPLEMENT;
 	}
-	
+
 	/* create join */
 	BuilderJoin *bjoin;
 	GdaSqlSelectJoin *join;
@@ -1204,7 +1211,7 @@ gda_sql_builder_select_join_targets (GdaSqlBuilder *builder, guint id,
 	join = (GdaSqlSelectJoin*) bjoin;
 	join->type = join_type;
 	join->position = right_pos;
-	
+
 	SqlPart *ep;
 	ep = get_part (builder, join_expr, GDA_SQL_ANY_EXPR);
 	if (ep)
@@ -1234,7 +1241,7 @@ gda_sql_builder_join_add_field (GdaSqlBuilder *builder, guint join_id, const gch
 	g_return_if_fail (GDA_IS_SQL_BUILDER (builder));
 	g_return_if_fail (builder->priv->main_stmt);
 	g_return_if_fail (field_name);
-	
+
 	if (builder->priv->main_stmt->stmt_type != GDA_SQL_STATEMENT_SELECT) {
 		g_warning (_("Wrong statement type"));
 		return;
@@ -1272,7 +1279,7 @@ gda_sql_builder_join_add_field (GdaSqlBuilder *builder, guint join_id, const gch
  * @collation_name: name of the collation to use when sorting, or %NULL
  *
  * Adds a new ORDER BY expression to a SELECT statement.
- * 
+ *
  * Since: 4.2
  */
 void
@@ -1295,7 +1302,7 @@ gda_sql_builder_select_order_by (GdaSqlBuilder *builder, guint expr_id,
 	if (!part)
 		return;
 	sel = (GdaSqlStatementSelect*) builder->priv->main_stmt->contents;
-	
+
 	sorder = gda_sql_select_order_new (GDA_SQL_ANY_PART (sel));
 	sorder->expr = (GdaSqlExpr*) use_part (part, GDA_SQL_ANY_PART (sorder));
 	sorder->asc = asc;
@@ -1304,6 +1311,186 @@ gda_sql_builder_select_order_by (GdaSqlBuilder *builder, guint expr_id,
 	sel->order_by = g_slist_append (sel->order_by, sorder);
 }
 
+/**
+ * gda_sql_builder_select_set_distinct
+ * @builder: a #GdaSqlBuilder object
+ * @distinct: set to %TRUE to have the DISTINCT requirement
+ * @expr_id: the ID of the DISTINCT ON expression, or %0 if no expression is to be used. It is ignored
+ *           if @distinct is %FALSE.
+ *
+ * Defines (if @distinct is %TRUE) or removes (if @distinct is %FALSE) a DISTINCT clause
+ * for a SELECT statement.
+ *
+ * If @distinct is %TRUE, then the ID of an expression can be specified as the @expr_id argument:
+ * if not %0, this is the expression used to apply the DISTINCT clause on (the resuting SQL
+ * will then usually be "... DISTINCT ON &lt;expression&gt;...").
+ *
+ * Since: 4.2
+ */
+void
+gda_sql_builder_select_set_distinct (GdaSqlBuilder *builder, gboolean distinct, guint expr_id)
+{
+	GdaSqlStatementSelect *sel;
+	SqlPart *part = NULL;
+
+	g_return_if_fail (GDA_IS_SQL_BUILDER (builder));
+
+	if (builder->priv->main_stmt->stmt_type != GDA_SQL_STATEMENT_SELECT) {
+		g_warning (_("Wrong statement type"));
+		return;
+	}
+
+	if (expr_id) {
+		part = get_part (builder, expr_id, GDA_SQL_ANY_EXPR);
+		if (!part)
+			return;
+	}
+
+	sel = (GdaSqlStatementSelect*) builder->priv->main_stmt->contents;
+	if (sel->distinct_expr) {
+		gda_sql_expr_free (sel->distinct_expr);
+		sel->distinct_expr = NULL;
+	}
+
+	if (distinct && part)
+		sel->distinct_expr = (GdaSqlExpr*) use_part (part, GDA_SQL_ANY_PART (sel));
+	sel->distinct = distinct;
+}
+
+/**
+ * gda_sql_builder_select_set_limit
+ * @builder: a #GdaSqlBuilder object
+ * @limit_count_expr_id: the ID of the LIMIT expression, or %0
+ * @limit_offset_expr_id: the ID of the OFFSET expression, or %0
+ *
+ * If @limit_count_expr_id is not %0, defines the maximum number of rows in the #GdaDataModel
+ * resulting from the execution of the built statement. In this case, the offset from which the
+ * rows must be collected can be defined by the @limit_offset_expr_id expression if not %0 (note that
+ * this feature may not be supported by all the database providers).
+ *
+ * If @limit_count_expr_id is %0, then removes any LIMIT which may have been imposed by a previous
+ * call to this method.
+ *
+ * Since: 4.2
+ */
+void
+gda_sql_builder_select_set_limit (GdaSqlBuilder *builder,
+				  guint limit_count_expr_id, guint limit_offset_expr_id)
+{
+	GdaSqlStatementSelect *sel;
+	SqlPart *part1 = NULL, *part2 = NULL;
+
+	g_return_if_fail (GDA_IS_SQL_BUILDER (builder));
+
+	if (builder->priv->main_stmt->stmt_type != GDA_SQL_STATEMENT_SELECT) {
+		g_warning (_("Wrong statement type"));
+		return;
+	}
+
+	if (limit_count_expr_id) {
+		part1 = get_part (builder, limit_count_expr_id, GDA_SQL_ANY_EXPR);
+		if (!part1)
+			return;
+	}
+	if (limit_offset_expr_id) {
+		part2 = get_part (builder, limit_offset_expr_id, GDA_SQL_ANY_EXPR);
+		if (!part2)
+			return;
+	}
+
+	sel = (GdaSqlStatementSelect*) builder->priv->main_stmt->contents;
+
+	if (sel->limit_count) {
+		gda_sql_expr_free (sel->limit_count);
+		sel->limit_count = NULL;
+	}
+	if (sel->limit_offset) {
+		gda_sql_expr_free (sel->limit_offset);
+		sel->limit_offset = NULL;
+	}
+	if (part1)
+		sel->limit_count = (GdaSqlExpr*) use_part (part1, GDA_SQL_ANY_PART (sel));
+	if (part2)
+		sel->limit_offset = (GdaSqlExpr*) use_part (part2, GDA_SQL_ANY_PART (sel));
+}
+
+/**
+ * gda_sql_builder_select_set_having
+ * @builder: a #GdaSqlBuilder object
+ * @cond_id: the ID of the expression to set as HAVING condition, or 0 to unset any previous HAVING condition
+ *
+ * Valid only for: SELECT statements
+ *
+ * Sets the HAVING condition of the statement
+ *
+ * Since: 4.2
+ */
+void
+gda_sql_builder_select_set_having (GdaSqlBuilder *builder, guint cond_id)
+{
+	GdaSqlStatementSelect *sel;
+
+	g_return_if_fail (GDA_IS_SQL_BUILDER (builder));
+
+	if (builder->priv->main_stmt->stmt_type != GDA_SQL_STATEMENT_SELECT) {
+		g_warning (_("Wrong statement type"));
+		return;
+	}
+
+	SqlPart *p = NULL;
+	if (cond_id > 0) {
+		p = get_part (builder, cond_id, GDA_SQL_ANY_EXPR);
+		if (!p)
+			return;
+	}
+
+	sel = (GdaSqlStatementSelect*) builder->priv->main_stmt->contents;
+	if (sel->having_cond)
+		gda_sql_expr_free (sel->having_cond);
+	sel->having_cond = (GdaSqlExpr*) use_part (p, GDA_SQL_ANY_PART (sel));
+}
+
+/**
+ * gda_sql_builder_select_group_by
+ * @builder: a #GdaSqlBuilder object
+ * @expr_id: the ID of the expression to set use in the GROUP BY clause, or 0 to unset any previous GROUP BY clause
+ *
+ * Valid only for: SELECT statements
+ *
+ * Adds the @expr_id expression to the GROUP BY clause's expressions list
+ *
+ * Since: 4.2
+ */
+void
+gda_sql_builder_select_group_by (GdaSqlBuilder *builder, guint expr_id)
+{
+	GdaSqlStatementSelect *sel;
+
+	g_return_if_fail (GDA_IS_SQL_BUILDER (builder));
+
+	if (builder->priv->main_stmt->stmt_type != GDA_SQL_STATEMENT_SELECT) {
+		g_warning (_("Wrong statement type"));
+		return;
+	}
+
+	SqlPart *p = NULL;
+	if (expr_id > 0) {
+		p = get_part (builder, expr_id, GDA_SQL_ANY_EXPR);
+		if (!p)
+			return;
+	}
+
+	sel = (GdaSqlStatementSelect*) builder->priv->main_stmt->contents;
+	if (p)
+		sel->group_by = g_slist_append (sel->group_by,
+						(GdaSqlExpr*) use_part (p, GDA_SQL_ANY_PART (sel)));
+	else if (sel->group_by) {
+		g_slist_foreach (sel->group_by, (GFunc)	gda_sql_expr_free, NULL);
+		g_slist_free (sel->group_by);
+		sel->group_by = NULL;
+	}
+}
+
 /**
  * gda_sql_builder_add_function
  * @builder: a #GdaSqlBuilder object
@@ -1414,7 +1601,7 @@ gda_sql_builder_add_sub_select (GdaSqlBuilder *builder, guint id, GdaSqlStatemen
 	g_return_val_if_fail (sqlst, 0);
 	g_return_val_if_fail ((sqlst->stmt_type == GDA_SQL_STATEMENT_SELECT) ||
 			      (sqlst->stmt_type == GDA_SQL_STATEMENT_COMPOUND), 0);
-	
+
 	GdaSqlExpr *expr;
 	expr = gda_sql_expr_new (NULL);
 	if (steal) {
@@ -1470,7 +1657,7 @@ gda_sql_builder_compound_set_type (GdaSqlBuilder *builder, GdaSqlStatementCompou
  * @steal: if %TRUE, then @sqlst will be "stolen" by @b and should not be used anymore
  *
  * Add a sub select to a COMPOUND statement
- * 
+ *
  * Since: 4.2
  */
 void
@@ -1523,7 +1710,7 @@ gda_sql_builder_add_case (GdaSqlBuilder *builder, guint id,
 	SqlPart *ptest, *pelse;
 	ptest = get_part (builder, test_expr, GDA_SQL_ANY_EXPR);
 	pelse = get_part (builder, else_expr, GDA_SQL_ANY_EXPR);
-	
+
 	GdaSqlExpr *expr;
 	expr = gda_sql_expr_new (NULL);
 
@@ -1532,7 +1719,7 @@ gda_sql_builder_add_case (GdaSqlBuilder *builder, guint id,
 		expr->case_s->base_expr = (GdaSqlExpr*) use_part (ptest, GDA_SQL_ANY_PART (expr->case_s));
 	if (pelse)
 		expr->case_s->else_expr = (GdaSqlExpr*) use_part (pelse, GDA_SQL_ANY_PART (expr->case_s));
-	
+
 	va_list ap;
 	guint id1;
 	va_start (ap, else_expr);
@@ -1557,7 +1744,7 @@ gda_sql_builder_add_case (GdaSqlBuilder *builder, guint id,
 	expr->case_s->when_expr_list = g_slist_reverse (expr->case_s->when_expr_list);
 	expr->case_s->then_expr_list = g_slist_reverse (expr->case_s->then_expr_list);
 	return add_part (builder, id, (GdaSqlAnyPart *) expr);
-	
+
  cleanups:
 	gda_sql_expr_free (expr);
 	return 0;
@@ -1592,7 +1779,7 @@ gda_sql_builder_add_case_v (GdaSqlBuilder *builder, guint id,
 	SqlPart *ptest, *pelse;
 	ptest = get_part (builder, test_expr, GDA_SQL_ANY_EXPR);
 	pelse = get_part (builder, else_expr, GDA_SQL_ANY_EXPR);
-	
+
 	GdaSqlExpr *expr;
 	expr = gda_sql_expr_new (NULL);
 
@@ -1601,7 +1788,7 @@ gda_sql_builder_add_case_v (GdaSqlBuilder *builder, guint id,
 		expr->case_s->base_expr = (GdaSqlExpr*) use_part (ptest, GDA_SQL_ANY_PART (expr->case_s));
 	if (pelse)
 		expr->case_s->else_expr = (GdaSqlExpr*) use_part (pelse, GDA_SQL_ANY_PART (expr->case_s));
-	
+
 	gint i;
 	for (i = 0; i < args_size; i++) {
 		SqlPart *pwhen, *pthen;
@@ -1619,8 +1806,58 @@ gda_sql_builder_add_case_v (GdaSqlBuilder *builder, guint id,
 	expr->case_s->when_expr_list = g_slist_reverse (expr->case_s->when_expr_list);
 	expr->case_s->then_expr_list = g_slist_reverse (expr->case_s->then_expr_list);
 	return add_part (builder, id, (GdaSqlAnyPart *) expr);
-	
+
  cleanups:
 	gda_sql_expr_free (expr);
 	return 0;
 }
+
+/**
+ * gda_sql_builder_export_expression
+ * @builder: a #GdaSqlBuilder object
+ * @id: the ID of the expression to be exported, (must be a valid ID in @builder, not %0)
+ *
+ * Exports a part managed by @builder as a new #GdaSqlExpr, which can represent any expression
+ * in a statement.
+ *
+ * Returns: a pointer to a new #GdaSqlExpr structure, free using gda_sql_expr_free() when not
+ * needed anymore. If the part with @id as ID cannot be found, the returned value is %NULL.
+ *
+ * Since: 4.2
+ */
+GdaSqlExpr *
+gda_sql_builder_export_expression (GdaSqlBuilder *builder, guint id)
+{
+	g_return_val_if_fail (GDA_IS_SQL_BUILDER (builder), NULL);
+	g_return_val_if_fail (builder->priv->main_stmt, NULL);
+
+	SqlPart *part;
+	part = get_part (builder, id, GDA_SQL_ANY_EXPR);
+	if (! part)
+		return NULL;
+	g_return_val_if_fail (part->part->type == GDA_SQL_ANY_EXPR, NULL);
+	return gda_sql_expr_copy ((GdaSqlExpr*) part->part);
+}
+
+/**
+ * gda_sql_builder_import_expression
+ * @builder: a #GdaSqlBuilder object
+ * @id: the requested ID, or 0 if to be determined by @builder
+ * @expr: a #GdaSqlExpr obtained using gda_sql_builder_export_expression()
+ *
+ * Imports the @expr into @builder.
+ *
+ * Returns: the ID of the new expression, or 0 if there was an error
+ *
+ * Since: 4.2
+ */
+guint
+gda_sql_builder_import_expression (GdaSqlBuilder *builder, guint id, GdaSqlExpr *expr)
+{
+	g_return_val_if_fail (GDA_IS_SQL_BUILDER (builder), 0);
+	g_return_val_if_fail (builder->priv->main_stmt, 0);
+	g_return_val_if_fail (expr, 0);
+
+	g_return_val_if_fail (GDA_SQL_ANY_PART (expr)->type == GDA_SQL_ANY_EXPR, 0);
+	return add_part (builder, id, (GdaSqlAnyPart *) gda_sql_expr_copy (expr));	
+}
