@@ -1,6 +1,6 @@
 /* gda-sql-builder.h
  *
- * Copyright (C) 2009 Vivien Malerba
+ * Copyright (C) 2009 - 2010 Vivien Malerba
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -30,6 +30,7 @@ G_BEGIN_DECLS
 #define GDA_SQL_BUILDER(obj)          G_TYPE_CHECK_INSTANCE_CAST (obj, gda_sql_builder_get_type(), GdaSqlBuilder)
 #define GDA_SQL_BUILDER_CLASS(klass)  G_TYPE_CHECK_CLASS_CAST (klass, gda_sql_builder_get_type (), GdaSqlBuilderClass)
 #define GDA_IS_SQL_BUILDER(obj)       G_TYPE_CHECK_INSTANCE_TYPE (obj, gda_sql_builder_get_type ())
+#define GDA_SQL_BUILDER_GET_CLASS(o)  (G_TYPE_INSTANCE_GET_CLASS ((o), GDA_TYPE_SQL_BUILDER, GdaSqlBuilderClass))
 
 typedef struct _GdaSqlBuilder        GdaSqlBuilder;
 typedef struct _GdaSqlBuilderClass   GdaSqlBuilderClass;
@@ -90,7 +91,7 @@ guint             gda_sql_builder_add_case (GdaSqlBuilder *builder, guint id,
 guint             gda_sql_builder_add_case_v (GdaSqlBuilder *builder, guint id,
 					      guint test_expr, guint else_expr,
 					      const guint *when_array, const guint *then_array, gint args_size);
-					    
+
 
 /* General Statement API */
 void              gda_sql_builder_add_field (GdaSqlBuilder *builder, const gchar *field_name, GType type, ...);
@@ -103,7 +104,7 @@ void              gda_sql_builder_set_where (GdaSqlBuilder *builder, guint cond_
 void              gda_sql_builder_add_field_id (GdaSqlBuilder *builder, guint field_id, guint value_id);
 
 /* SELECT Statement API */
-void              gda_sql_builder_select_add_field (GdaSqlBuilder *builder, const gchar *field_name,
+guint             gda_sql_builder_select_add_field (GdaSqlBuilder *builder, const gchar *field_name,
 						    const gchar *table_name, const gchar *alias);
 guint             gda_sql_builder_select_add_target (GdaSqlBuilder *builder, const gchar *table_name, const gchar *alias);
 guint             gda_sql_builder_select_add_target_id (GdaSqlBuilder *builder, guint id,
@@ -115,11 +116,22 @@ guint             gda_sql_builder_select_join_targets (GdaSqlBuilder *builder, g
 void              gda_sql_builder_join_add_field (GdaSqlBuilder *builder, guint join_id, const gchar *field_name);
 void              gda_sql_builder_select_order_by (GdaSqlBuilder *builder, guint expr_id,
 						   gboolean asc, const gchar *collation_name);
+void              gda_sql_builder_select_set_distinct (GdaSqlBuilder *builder,
+						       gboolean distinct, guint expr_id);
+void              gda_sql_builder_select_set_limit (GdaSqlBuilder *builder,
+						    guint limit_count_expr_id, guint limit_offest_expr_id);
+
+void              gda_sql_builder_select_set_having (GdaSqlBuilder *builder, guint cond_id);
+void              gda_sql_builder_select_group_by (GdaSqlBuilder *builder, guint expr_id);
 
 /* COMPOUND SELECT Statement API */
 void              gda_sql_builder_compound_set_type (GdaSqlBuilder *builder, GdaSqlStatementCompoundType compound_type);
 void              gda_sql_builder_compound_add_sub_select (GdaSqlBuilder *builder, GdaSqlStatement *sqlst, gboolean steal);
 
+/* import/Export API */
+GdaSqlExpr       *gda_sql_builder_export_expression (GdaSqlBuilder *builder, guint id);
+guint             gda_sql_builder_import_expression (GdaSqlBuilder *builder, guint id, GdaSqlExpr *expr);
+
 G_END_DECLS
 
 #endif
