@@ -1,5 +1,5 @@
 /* GDA common library
- * Copyright (C) 1998 - 2008 The GNOME Foundation.
+ * Copyright (C) 1998 - 2009 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
@@ -48,6 +48,8 @@ guint        gda_identifier_hash (const gchar *id);
 gboolean     gda_identifier_equal (const gchar *id1, const gchar *id2);
 gchar      **gda_completion_list_get (GdaConnection *cnc, const gchar *sql, gint start, gint end);
 gchar      **gda_sql_identifier_split (const gchar *id);
+gchar       *gda_sql_identifier_quote (const gchar *id, GdaConnection *cnc, GdaServerProvider *prov,
+				       gboolean meta_store_convention, gboolean force_quotes);
 
 /*
  * Param & model utilities
@@ -70,6 +72,11 @@ gchar       *gda_alphanum_to_text (gchar *text);
  */
 GdaSqlExpr      *gda_compute_unique_table_row_condition (GdaSqlStatementSelect *stsel, GdaMetaTable *mtable, 
 							 gboolean require_pk, GError **error);
+GdaSqlExpr       *gda_compute_unique_table_row_condition_with_cnc (GdaConnection *cnc,
+								   GdaSqlStatementSelect *stsel,
+								   GdaMetaTable *mtable, gboolean require_pk,
+								   GError **error);
+
 gboolean         gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gboolean require_pk, 
 					     GdaStatement **insert_stmt, GdaStatement **update_stmt, GdaStatement **delete_stmt, 
 					     GError **error);
