@@ -1,5 +1,5 @@
 /* GDA Jdbc provider
- * Copyright (C) 2008 The GNOME Foundation.
+ * Copyright (C) 2008 - 2009 The GNOME Foundation.
  *
  * AUTHORS:
  *      Vivien Malerba <malerba@gnome-db.org>
@@ -88,7 +88,7 @@ static gboolean            gda_jdbc_provider_rollback_savepoint (GdaServerProvid
 static gboolean            gda_jdbc_provider_delete_savepoint (GdaServerProvider *provider, GdaConnection *cnc,
 							       const gchar *name, GError **error);
 
-/* information retreival */
+/* information retrieval */
 static const gchar        *gda_jdbc_provider_get_version (GdaServerProvider *provider);
 static gboolean            gda_jdbc_provider_supports_feature (GdaServerProvider *provider, GdaConnection *cnc,
 							       GdaConnectionFeature feature);
@@ -187,7 +187,8 @@ gda_jdbc_provider_class_init (GdaJdbcProviderClass *klass)
         provider_class->delete_savepoint = gda_jdbc_provider_delete_savepoint;
 
 	provider_class->create_parser = NULL;
-	provider_class->statement_to_sql = gda_jdbc_provider_statement_to_sql;
+	provider_class->statement_to_sql = NULL; /* don't use gda_jdbc_provider_statement_to_sql()
+						  * because it only calls gda_statement_to_sql_extended() */
 	provider_class->statement_prepare = gda_jdbc_provider_statement_prepare;
 	provider_class->statement_execute = gda_jdbc_provider_statement_execute;
 
@@ -238,6 +239,10 @@ gda_jdbc_provider_class_init (GdaJdbcProviderClass *klass)
 	provider_class->meta_funcs.routine_col = _gda_jdbc_meta_routine_col;
 	provider_class->meta_funcs._routine_par = _gda_jdbc_meta__routine_par;
 	provider_class->meta_funcs.routine_par = _gda_jdbc_meta_routine_par;
+	provider_class->meta_funcs._indexes_tab = _gda_jdbc_meta__indexes_tab;
+        provider_class->meta_funcs.indexes_tab = _gda_jdbc_meta_indexes_tab;
+        provider_class->meta_funcs._index_cols = _gda_jdbc_meta__index_cols;
+        provider_class->meta_funcs.index_cols = _gda_jdbc_meta_index_cols;
 
 	/* distributed transactions: if not supported, then provider_class->xa_funcs should be set to NULL */
 	provider_class->xa_funcs = g_new0 (GdaServerProviderXa, 1);
@@ -517,7 +522,7 @@ gda_jdbc_provider_supports_operation (GdaServerProvider *provider, GdaConnection
  * Create operation request
  *
  * Creates a #GdaServerOperation. The following code is generic and should only be changed
- * if some further initialization is required, or if operation's contents is dependant on @cnc
+ * if some further initialization is required, or if operation's contents is dependent on @cnc
  */
 static GdaServerOperation *
 gda_jdbc_provider_create_operation (GdaServerProvider *provider, GdaConnection *cnc,
@@ -1006,6 +1011,9 @@ gda_jdbc_provider_get_default_dbms_type (GdaServerProvider *provider, GdaConnect
  * The implementation show here simply calls gda_statement_to_sql_extended() but the rendering
  * can be specialized to the database's SQL dialect, see the implementation of gda_statement_to_sql_extended()
  * and SQLite's specialized rendering for more details
+ *
+ * NOTE: This implementation MUST NOT call gda_statement_to_sql_extended() if it is
+ *       the GdaServerProvider::statement_to_sql() virtual method's implementation
  */
 static gchar *
 gda_jdbc_provider_statement_to_sql (GdaServerProvider *provider, GdaConnection *cnc,
