@@ -34,6 +34,7 @@
 #include <libgda/binreloc/gda-binreloc.h>
 #include <libgda/gda-statement-extra.h>
 #include <sql-parser/gda-sql-parser.h>
+#include <libgda/gda-blob-op.h>
 #include "gda-mysql.h"
 #include "gda-mysql-provider.h"
 #include "gda-mysql-recordset.h"
@@ -42,6 +43,7 @@
 
 #include "gda-mysql-util.h"
 #include "gda-mysql-parser.h"
+#include "gda-mysql-handler-boolean.h"
 
 #define _GDA_PSTMT(x) ((GdaPStmt*)(x))
 
@@ -1276,8 +1278,26 @@ gda_mysql_provider_get_data_handler (GdaServerProvider  *provider,
 	else if ((type == GDA_TYPE_TIME) ||
 		 (type == GDA_TYPE_TIMESTAMP) ||
 		 (type == G_TYPE_DATE)) {
-		TO_IMPLEMENT; /* define data handlers for these types */
-		dh = NULL;
+		dh = gda_server_provider_handler_find (provider, NULL, type, NULL);
+                if (!dh) {
+                        dh = gda_handler_time_new ();
+                        gda_handler_time_set_sql_spec ((GdaHandlerTime *) dh, G_DATE_YEAR,
+                                                       G_DATE_MONTH, G_DATE_DAY, '-', FALSE);
+                        gda_server_provider_handler_declare (provider, dh, NULL, G_TYPE_DATE, NULL);
+                        gda_server_provider_handler_declare (provider, dh, NULL, GDA_TYPE_TIME, NULL);
+                        gda_server_provider_handler_declare (provider, dh, NULL, GDA_TYPE_TIMESTAMP, NULL);
+                        g_object_unref (dh);
+                }
+	}
+	else if (type == G_TYPE_BOOLEAN){
+		dh = gda_server_provider_handler_find (provider, cnc, type, NULL);
+                if (!dh) {
+                        dh = gda_mysql_handler_boolean_new ();
+                        if (dh) {
+                                gda_server_provider_handler_declare (provider, dh, cnc, G_TYPE_BOOLEAN, NULL);
+                                g_object_unref (dh);
+                        }
+                }
 	}
 	else
 		dh = gda_server_provider_get_data_handler_default (provider, cnc, type, dbms_type);
@@ -1371,6 +1391,9 @@ gda_mysql_provider_create_parser (GdaServerProvider  *provider,
  * This method renders a #GdaStatement into its SQL representation.
  */
 static gchar *mysql_render_function (GdaSqlFunction *func, GdaSqlRenderingContext *context, GError **error);
+static gchar *mysql_render_expr (GdaSqlExpr *expr, GdaSqlRenderingContext *context, 
+				 gboolean *is_default, gboolean *is_null,
+				 GError **error);
 static gchar *
 gda_mysql_provider_statement_to_sql (GdaServerProvider    *provider,
 				     GdaConnection        *cnc,
@@ -1390,15 +1413,14 @@ gda_mysql_provider_statement_to_sql (GdaServerProvider    *provider,
         }
 
         memset (&context, 0, sizeof (context));
+	context.provider = provider;
+	context.cnc = cnc;
         context.params = params;
         context.flags = flags;
 	context.render_function = (GdaSqlRenderingFunc) mysql_render_function; /* don't put any space between function name
 										* and the opening parenthesis, see
 										* http://blog.152.org/2009/12/mysql-error-1305-function-xxx-does-not.html */
-	if (cnc) {
-		context.cnc = cnc;
-		context.provider = gda_connection_get_provider (cnc);
-	}
+	context.render_expr = mysql_render_expr; /* render "FALSE" as 0 and TRUE as 1 */
 
         str = gda_statement_to_sql_real (stmt, &context, error);
 
@@ -1450,6 +1472,136 @@ mysql_render_function (GdaSqlFunction *func, GdaSqlRenderingContext *context, GE
 	return NULL;
 }
 
+static gchar *
+mysql_render_expr (GdaSqlExpr *expr, GdaSqlRenderingContext *context, 
+		   gboolean *is_default, gboolean *is_null,
+		   GError **error)
+{
+	GString *string;
+	gchar *str = NULL;
+
+	g_return_val_if_fail (expr, NULL);
+	g_return_val_if_fail (GDA_SQL_ANY_PART (expr)->type == GDA_SQL_ANY_EXPR, NULL);
+
+	if (is_default)
+		*is_default = FALSE;
+	if (is_null)
+		*is_null = FALSE;
+
+	/* can't have: 
+	 *  - expr->cast_as && expr->param_spec 
+	 */
+	if (!gda_sql_any_part_check_structure (GDA_SQL_ANY_PART (expr), error)) return NULL;
+
+	string = g_string_new ("");
+	if (expr->param_spec) {
+		str = context->render_param_spec (expr->param_spec, expr, context, is_default, is_null, error);
+		if (!str) goto err;
+	}
+	else if (expr->value) {
+		if (expr->value_is_ident && (G_VALUE_TYPE (expr->value) == G_TYPE_STRING) &&
+		    g_value_get_string (expr->value)) {
+			gchar **ids_array;
+			gint i;
+			GString *string = NULL;
+			GdaConnectionOptions cncoptions = 0;
+			if (context->cnc)
+				g_object_get (G_OBJECT (context->cnc), "options", &cncoptions, NULL);
+
+			ids_array = gda_sql_identifier_split (g_value_get_string (expr->value));
+			if (!ids_array) 
+				str = g_value_dup_string (expr->value);
+			else if (!(ids_array[0])) goto err;
+			else {
+				for (i = 0; ids_array[i]; i++) {
+					gchar *tmp;
+					if (!string)
+						string = g_string_new ("");
+					else
+						g_string_append_c (string, '.');
+					tmp = gda_sql_identifier_quote (ids_array[i], context->cnc,
+									context->provider, FALSE,
+					   cncoptions & GDA_CONNECTION_OPTIONS_SQL_IDENTIFIERS_CASE_SENSITIVE);
+					g_string_append (string, tmp);
+					g_free (tmp);
+				}
+				g_strfreev (ids_array);
+				str = g_string_free (string, FALSE);
+			}
+		}
+		else {
+			str = gda_value_stringify (expr->value);
+			if (!str) goto err;
+			if (is_null && gda_value_is_null (expr->value))
+				*is_null = TRUE;
+			else if (is_default && (G_VALUE_TYPE (expr->value) == G_TYPE_STRING) && 
+				 !g_ascii_strcasecmp (g_value_get_string (expr->value), "default"))
+				*is_default = TRUE;
+			else if (!g_ascii_strcasecmp (str, "FALSE")) {
+				g_free (str);
+				str = g_strdup ("0");
+			}
+			else if (!g_ascii_strcasecmp (str, "TRUE")) {
+				g_free (str);
+				str = g_strdup ("1");
+			}
+		}
+	}
+	else if (expr->func) {
+		str = context->render_function (GDA_SQL_ANY_PART (expr->func), context, error);
+		if (!str) goto err;
+	}
+	else if (expr->cond) {
+		str = context->render_operation (GDA_SQL_ANY_PART (expr->cond), context, error);
+		if (!str) goto err;
+	}
+	else if (expr->select) {
+		gchar *str1;
+		if (GDA_SQL_ANY_PART (expr->select)->type == GDA_SQL_ANY_STMT_SELECT)
+			str1 = context->render_select (GDA_SQL_ANY_PART (expr->select), context, error);
+		else
+			str1 = context->render_compound (GDA_SQL_ANY_PART (expr->select), context, error);
+		if (!str1) goto err;
+
+		if (! GDA_SQL_ANY_PART (expr)->parent ||
+		    (GDA_SQL_ANY_PART (expr)->parent->type != GDA_SQL_ANY_SQL_FUNCTION)) {
+			str = g_strdup_printf ("(%s)", str1);
+			g_free (str1);
+		}
+		else
+			str = str1;
+	}
+	else if (expr->case_s) {
+		str = context->render_case (GDA_SQL_ANY_PART (expr->case_s), context, error);
+		if (!str) goto err;
+	}
+	else {
+		if (is_null)
+			*is_null = TRUE;
+		str = g_strdup ("NULL");
+	}
+
+	if (!str) {
+		/* TO REMOVE */
+		str = g_strdup ("[...]");
+	}
+
+	if (expr->cast_as) 
+		g_string_append_printf (string, "CAST (%s AS %s)", str, expr->cast_as);
+	else
+		g_string_append (string, str);
+	g_free (str);
+
+	str = string->str;
+	g_string_free (string, FALSE);
+	return str;
+
+ err:
+	g_string_free (string, TRUE);
+	return NULL;
+}
+
+
 static GdaMysqlPStmt *
 real_prepare (GdaServerProvider *provider, GdaConnection *cnc, GdaStatement *stmt, GError **error)
 {
@@ -1603,6 +1755,180 @@ prepare_stmt_simple (MysqlConnectionData  *cdata,
 	return ps;
 }
 
+static GdaSet *
+make_last_inserted_set (GdaConnection *cnc, GdaStatement *stmt, my_ulonglong last_id)
+{
+	GError *lerror = NULL;
+
+	/* analyze @stmt */
+	GdaSqlStatement *sql_insert;
+	GdaSqlStatementInsert *insert;
+	if (gda_statement_get_statement_type (stmt) != GDA_SQL_STATEMENT_INSERT)
+		/* unable to compute anything */
+		return NULL;
+
+	g_object_get (G_OBJECT (stmt), "structure", &sql_insert, NULL);
+	g_assert (sql_insert);
+	insert = (GdaSqlStatementInsert *) sql_insert->contents;
+
+	/* find the name of the 1st column which has the AUTO_INCREMENT */
+	gchar *autoinc_colname = NULL;
+	gint rc;
+	gchar *sql;
+	MysqlConnectionData *cdata;
+	cdata = (MysqlConnectionData*) gda_connection_internal_get_provider_data (cnc);
+	if (!cdata) 
+		return NULL;
+	sql = g_strdup_printf ("DESCRIBE %s", insert->table->table_name);
+	rc = gda_mysql_real_query_wrap (cnc, cdata->mysql, sql, strlen (sql));
+	g_free (sql);
+	if (!rc) {
+		MYSQL_RES *result;
+		result = mysql_store_result (cdata->mysql);
+		if (result) {
+			MYSQL_ROW row;
+			unsigned int nfields;
+			nfields = mysql_num_fields (result);
+			while ((row = mysql_fetch_row(result))) {
+				if (row [nfields-1] &&
+				    !g_ascii_strcasecmp (row [nfields-1], "auto_increment")) {
+					autoinc_colname = g_strdup (row [0]);
+					break;
+				}
+			}
+			mysql_free_result (result);
+		}
+	}
+	if (! autoinc_colname) {
+		gda_sql_statement_free (sql_insert);
+		return NULL;
+	}
+
+	/* build corresponding SELECT statement */
+	GdaSqlStatementSelect *select;
+        GdaSqlSelectTarget *target;
+        GdaSqlStatement *sql_statement = gda_sql_statement_new (GDA_SQL_STATEMENT_SELECT);
+
+	select = g_new0 (GdaSqlStatementSelect, 1);
+        GDA_SQL_ANY_PART (select)->type = GDA_SQL_ANY_STMT_SELECT;
+        sql_statement->contents = select;
+
+	/* FROM */
+        select->from = gda_sql_select_from_new (GDA_SQL_ANY_PART (select));
+        target = gda_sql_select_target_new (GDA_SQL_ANY_PART (select->from));
+        gda_sql_select_from_take_new_target (select->from, target);
+
+	/* Filling in the target */
+        GValue *value;
+        g_value_set_string ((value = gda_value_new (G_TYPE_STRING)), insert->table->table_name);
+        gda_sql_select_target_take_table_name (target, value);
+	gda_sql_statement_free (sql_insert);
+
+	/* selected fields */
+        GdaSqlSelectField *field;
+        GSList *fields_list = NULL;
+
+	field = gda_sql_select_field_new (GDA_SQL_ANY_PART (select));
+	g_value_set_string ((value = gda_value_new (G_TYPE_STRING)), "*");
+	gda_sql_select_field_take_star_value (field, value);
+	fields_list = g_slist_append (fields_list, field);
+
+	gda_sql_statement_select_take_expr_list (sql_statement, fields_list);
+
+	/* WHERE */
+        GdaSqlExpr *where, *expr;
+	GdaSqlOperation *cond;
+	where = gda_sql_expr_new (GDA_SQL_ANY_PART (select));
+	cond = gda_sql_operation_new (GDA_SQL_ANY_PART (where));
+	where->cond = cond;
+	cond->operator_type = GDA_SQL_OPERATOR_TYPE_EQ;
+	expr = gda_sql_expr_new (GDA_SQL_ANY_PART (cond));
+	g_value_take_string ((value = gda_value_new (G_TYPE_STRING)), autoinc_colname);
+	expr->value = value;
+	cond->operands = g_slist_append (NULL, expr);
+	gchar *str;
+	str = g_strdup_printf ("%llu", last_id);
+	expr = gda_sql_expr_new (GDA_SQL_ANY_PART (cond));
+	g_value_take_string ((value = gda_value_new (G_TYPE_STRING)), str);
+	expr->value = value;
+	cond->operands = g_slist_append (cond->operands, expr);
+
+	gda_sql_statement_select_take_where_cond (sql_statement, where);
+
+	if (gda_sql_statement_check_structure (sql_statement, &lerror) == FALSE) {
+                g_warning (_("Can't build SELECT statement to get last inserted row: %s)"),
+			     lerror && lerror->message ? lerror->message : _("No detail"));
+		if (lerror)
+			g_error_free (lerror);
+                gda_sql_statement_free (sql_statement);
+                return NULL;
+        }
+
+	/* execute SELECT statement */
+	GdaDataModel *model;
+	GdaStatement *statement = g_object_new (GDA_TYPE_STATEMENT, "structure", sql_statement, NULL);
+        gda_sql_statement_free (sql_statement);
+        model = gda_connection_statement_execute_select (cnc, statement, NULL, &lerror);
+	g_object_unref (statement);
+        if (!model) {
+                g_warning (_("Can't execute SELECT statement to get last inserted row: %s"),
+			   lerror && lerror->message ? lerror->message : _("No detail"));
+		if (lerror)
+			g_error_free (lerror);
+		return NULL;
+        }
+	else {
+		GdaSet *set = NULL;
+		GSList *holders = NULL;
+		gint nrows, ncols, i;
+
+		nrows = gda_data_model_get_n_rows (model);
+		if (nrows <= 0) {
+			g_warning (_("SELECT statement to get last inserted row did not return any row"));
+			return NULL;
+		}
+		else if (nrows > 1) {
+			g_warning (_("SELECT statement to get last inserted row returned too many (%d) rows"),
+				   nrows);
+			return NULL;
+		}
+		ncols = gda_data_model_get_n_columns (model);
+		for (i = 0; i < ncols; i++) {
+			GdaHolder *h;
+			GdaColumn *col;
+			gchar *id;
+			const GValue *cvalue;
+
+			col = gda_data_model_describe_column (model, i);
+			h = gda_holder_new (gda_column_get_g_type (col));
+			id = g_strdup_printf ("+%d", i);
+			g_object_set (G_OBJECT (h), "id", id, "not-null", FALSE,
+				      "name", gda_column_get_name (col), NULL);
+			g_free (id);
+			cvalue = gda_data_model_get_value_at (model, i, 0, NULL);
+			if (!cvalue || !gda_holder_set_value (h, cvalue, NULL)) {
+				if (holders) {
+					g_slist_foreach (holders, (GFunc) g_object_unref, NULL);
+					g_slist_free (holders);
+					holders = NULL;
+				}
+				break;
+			}
+			holders = g_slist_prepend (holders, h);
+		}
+		g_object_unref (model);
+
+		if (holders) {
+			holders = g_slist_reverse (holders);
+			set = gda_set_new (holders);
+			g_slist_foreach (holders, (GFunc) g_object_unref, NULL);
+			g_slist_free (holders);
+		}
+
+		return set;
+	}
+}
+
 static void
 free_bind_param_data (GSList *mem_to_free)
 {
@@ -1787,7 +2113,7 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 			mysql_bind_param[i].buffer_type = MYSQL_TYPE_NULL;
 			mysql_bind_param[i].is_null = (my_bool*)1;
 		}
-		else if (G_VALUE_TYPE(value) == GDA_TYPE_TIMESTAMP) {
+		else if (G_VALUE_TYPE (value) == GDA_TYPE_TIMESTAMP) {
 			const GdaTimestamp *ts;
 
 			ts = gda_value_get_timestamp (value);
@@ -1812,7 +2138,7 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 				mysql_bind_param[i].buffer_length = sizeof (MYSQL_TIME);
 			}
 		}
-		else if (G_VALUE_TYPE(value) == GDA_TYPE_TIME) {
+		else if (G_VALUE_TYPE (value) == GDA_TYPE_TIME) {
 			const GdaTime *ts;
 
 			ts = gda_value_get_time (value);
@@ -1834,7 +2160,7 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 				mysql_bind_param[i].buffer_length = sizeof (MYSQL_TIME);
 			}
 		}
-		else if (G_VALUE_TYPE(value) == G_TYPE_DATE) {
+		else if (G_VALUE_TYPE (value) == G_TYPE_DATE) {
 			const GDate *ts;
 
 			ts = (GDate*) g_value_get_boxed (value);
@@ -1846,16 +2172,16 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 				MYSQL_TIME *mtime;
 				mtime = g_new0 (MYSQL_TIME, 1);
 				mem_to_free = g_slist_prepend (mem_to_free, mtime);
-				mtime->year = ts->year;
-				mtime->month = ts->month;
-				mtime->day = ts->day;
+				mtime->year = g_date_get_year (ts);
+				mtime->month = g_date_get_month (ts);
+				mtime->day = g_date_get_day (ts);
 
 				mysql_bind_param[i].buffer_type= MYSQL_TYPE_DATE;
 				mysql_bind_param[i].buffer= (char *)mtime;
 				mysql_bind_param[i].buffer_length = sizeof (MYSQL_TIME);
 			}
 		}
-		else if (G_VALUE_TYPE(value) == G_TYPE_STRING) {
+		else if (G_VALUE_TYPE (value) == G_TYPE_STRING) {
 			const gchar *str;
 			str = g_value_get_string (value);
 			if (!str) {
@@ -1869,15 +2195,75 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 				mysql_bind_param[i].length = NULL; /* str is 0 terminated */
 			}
 		}
+		else if (G_VALUE_TYPE (value) == GDA_TYPE_BLOB) {
+			const GdaBinary *bin = NULL;
+			GdaBlob *blob = (GdaBlob*) gda_value_get_blob (value);
+
+			bin = ((GdaBinary*) blob);
+			if (!bin) {
+				mysql_bind_param[i].buffer_type = MYSQL_TYPE_NULL;
+				mysql_bind_param[i].is_null = (my_bool*)1;
+			}
+			else {
+				gchar *str = NULL;
+				gulong blob_len;
+				if (blob->op) {
+					blob_len = gda_blob_op_get_length (blob->op);
+					if ((blob_len != bin->binary_length) &&
+					    ! gda_blob_op_read_all (blob->op, blob)) {
+						/* force reading the complete BLOB into memory */
+						str = _("Can't read whole BLOB into memory");
+					}
+				}
+				else
+					blob_len = bin->binary_length;
+				if (blob_len < 0)
+					str = _("Can't get BLOB's length");
+				else if (blob_len >= G_MAXINT)
+					str = _("BLOB is too big");
+				
+				if (str) {
+					event = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
+					gda_connection_event_set_description (event, str);
+					g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+						     GDA_SERVER_PROVIDER_DATA_ERROR, "%s", str);
+					break;
+				}
+				
+				else {
+					mysql_bind_param[i].buffer_type= MYSQL_TYPE_BLOB;
+					mysql_bind_param[i].buffer= (char *) bin->data;
+					mysql_bind_param[i].buffer_length = bin->binary_length;
+					mysql_bind_param[i].length = NULL;
+				}
+			}
+		}
+		else if (G_VALUE_TYPE (value) == GDA_TYPE_BINARY) {
+			const GdaBinary *bin;
+			bin = gda_value_get_binary (value);
+			if (!bin) {
+				mysql_bind_param[i].buffer_type = MYSQL_TYPE_NULL;
+				mysql_bind_param[i].is_null = (my_bool*)1;
+			}
+			else {
+				mysql_bind_param[i].buffer_type= MYSQL_TYPE_BLOB;
+				mysql_bind_param[i].buffer= (char *) bin->data;
+				mysql_bind_param[i].buffer_length = bin->binary_length;
+				mysql_bind_param[i].length = NULL;
+			}
+		}
 		else {
 			gchar *str;
 			GdaDataHandler *data_handler =
 				gda_server_provider_get_data_handler_g_type (provider, cnc, 
 									     G_VALUE_TYPE (value));
 			if (data_handler == NULL) {
-				TO_IMPLEMENT;
 				/* there is an error here */
-				str = NULL;
+				event = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
+				gda_connection_event_set_description (event, str);
+				g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+					     GDA_SERVER_PROVIDER_DATA_ERROR, "%s", str);
+				break;
 			}
 			else {
 				str = gda_data_handler_get_str_from_value (data_handler, value);
@@ -1890,7 +2276,7 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 		}
 	}
 		
-	if (mysql_bind_param && mysql_stmt_bind_param (ps->mysql_stmt, mysql_bind_param)) {
+	if (!event && mysql_bind_param && mysql_stmt_bind_param (ps->mysql_stmt, mysql_bind_param)) {
 		//g_warning ("mysql_stmt_bind_param failed: %s\n", mysql_stmt_error (ps->mysql_stmt));
 		event = _gda_mysql_make_error (cnc, cdata->mysql, ps->mysql_stmt, error);
 	}
@@ -2010,6 +2396,14 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 			else {
 				return_value = (GObject *) gda_data_model_array_new (0);
 			}
+
+			if (last_inserted_row) {
+				my_ulonglong last_row;
+				last_row = mysql_insert_id (cdata->mysql);
+				if (last_row)
+					*last_inserted_row = make_last_inserted_set (cnc, stmt, last_row);
+			}
+
 		}
 	}
 	g_object_unref (ps);
