@@ -39,6 +39,7 @@ typedef struct {
 	AuthDialogConnection ext;
 	GdaDsnInfo  cncinfo;
 	GtkWidget  *auth_widget;
+	GString    *auth_string;
 
 	GdaThreadWrapper *wrapper;
 	guint jobid;
@@ -53,6 +54,8 @@ auth_data_free (AuthData *ad)
 	g_free (ad->cncinfo.cnc_string);
 	g_free (ad->cncinfo.auth_string);
 	g_free (ad->ext.cnc_string);
+	if (ad->auth_string)
+		g_string_free (ad->auth_string, TRUE);
 
 	g_object_unref (ad->wrapper);
 	if (ad->ext.cnc_open_error)
@@ -134,6 +137,7 @@ auth_dialog_init (AuthDialog *dialog)
 {
 	GtkWidget *label, *hbox, *wid;
 	char *markup, *str;
+	GtkWidget *dcontents;
 
 	dialog->priv = g_new0 (AuthDialogPrivate, 1);
 
@@ -143,7 +147,12 @@ auth_dialog_init (AuthDialog *dialog)
 				GTK_STOCK_CANCEL,
 				GTK_RESPONSE_REJECT, NULL);
 
-	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 5);
+#if GTK_CHECK_VERSION(2,18,0)
+	dcontents = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+#else
+	dcontents = GTK_DIALOG (dialog)->vbox;
+#endif
+	gtk_box_set_spacing (GTK_BOX (dcontents), 5);
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT, TRUE);
 
 	str = gda_gbr_get_file_path (GDA_DATA_DIR, LIBGDA_ABI_NAME, "pixmaps", "gda-browser-auth.png", NULL);
@@ -153,7 +162,7 @@ auth_dialog_init (AuthDialog *dialog)
 	/* label and spinner */
 	hbox = gtk_hbox_new (FALSE, 0); 
 	gtk_container_set_border_width (GTK_CONTAINER (hbox), 10);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (dcontents), hbox, FALSE, FALSE, 0);
 	
 	str = gda_gbr_get_file_path (GDA_DATA_DIR, LIBGDA_ABI_NAME, "pixmaps", "gda-browser-auth-big.png", NULL);
 	wid = gtk_image_new_from_file (str);
@@ -223,12 +232,12 @@ sub_thread_open_cnc (AuthData *ad, GError **error)
 	GdaConnection *cnc;
 	GdaDsnInfo *info = &(ad->cncinfo);
 	if (info->name)
-		cnc = gda_connection_open_from_dsn (info->name, info->auth_string,
+		cnc = gda_connection_open_from_dsn (info->name, ad->auth_string ? ad->auth_string->str : NULL,
 						    GDA_CONNECTION_OPTIONS_THREAD_SAFE,
 						    error);
 	else
 		cnc = gda_connection_open_from_string (info->provider, info->cnc_string,
-						       info->auth_string,
+						       ad->auth_string ? ad->auth_string->str : NULL,
 						       GDA_CONNECTION_OPTIONS_THREAD_SAFE,
 						       error);
 	return cnc;
@@ -324,6 +333,7 @@ auth_dialog_add_cnc_string (AuthDialog *dialog, const gchar *cnc_string, GError
 	ad = g_new0 (AuthData, 1);
 	ad->wrapper = gda_thread_wrapper_new ();
 	ad->ext.cnc_string = g_strdup (cnc_string);
+	ad->auth_string = NULL;
 	info = gda_config_get_dsn_info (real_cnc);
         if (info && !real_provider) {
 		ad->cncinfo.name = g_strdup (info->name);
@@ -390,25 +400,47 @@ auth_dialog_add_cnc_string (AuthDialog *dialog, const gchar *cnc_string, GError
 
                 set = gda_set_copy (pinfo->auth_params);
                 ad->auth_widget = gdaui_basic_form_new (set);
-                /*g_signal_connect (G_OBJECT (ad->auth_widget), "param-changed",
+                /*g_signal_connect (G_OBJECT (ad->auth_widget), "holder-changed",
 		  G_CALLBACK (auth_form_changed), dialog);*/
                 g_object_unref (set);
 
 		/* add widget */
 		GtkWidget *hbox, *label;
-		gchar *str;
+		gchar *str, *tmp, *ptr;
+		GtkWidget *dcontents;
+
+#if GTK_CHECK_VERSION(2,18,0)
+		dcontents = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+#else
+		dcontents = GTK_DIALOG (dialog)->vbox;
+#endif
 
 		label = gtk_label_new ("");
-		str = g_strdup_printf ("<b>%s: %s</b>\n%s", _("For connection"), ad->ext.cnc_string,
+		tmp = g_strdup (ad->ext.cnc_string);
+		for (ptr = tmp; *ptr; ptr++) {
+			if (*ptr == ':') {
+				/* remove everything up to the '@' */
+				gchar *ptr2;
+				for (ptr2 = ptr+1; *ptr2; ptr2++) {
+					if (*ptr2 == '@') {
+						memmove (ptr, ptr2, strlen (ptr2) + 1);
+						break;
+					}
+				}
+				break;
+			}
+		}
+		str = g_strdup_printf ("<b>%s: %s</b>\n%s", _("For connection"), tmp,
 				       _("enter authentication information"));
+		g_free (tmp);
 		gtk_label_set_markup (GTK_LABEL (label), str);
 		gtk_misc_set_alignment (GTK_MISC (label), 0., -1);
 		g_free (str);
-		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), label, FALSE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (dcontents), label, FALSE, FALSE, 0);
 		gtk_widget_show (label);
 
 		hbox = gtk_hbox_new (FALSE, 0); /* HIG */
-		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, TRUE, TRUE, 0);
+		gtk_box_pack_start (GTK_BOX (dcontents), hbox, TRUE, TRUE, 0);
 		label = gtk_label_new ("      ");
 		gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 		gtk_box_pack_start (GTK_BOX (hbox), ad->auth_widget, TRUE, TRUE, 0);
@@ -524,6 +556,39 @@ auth_dialog_run (AuthDialog *dialog)
 				AuthData *ad;
 				ad = (AuthData *) list->data;
 				if (ad->auth_widget && !ad->jobid) {
+					GSList *plist;
+					GdaSet *set;
+					set = gdaui_basic_form_get_data_set (GDAUI_BASIC_FORM (ad->auth_widget));
+					if (ad->auth_string) {
+						g_string_free (ad->auth_string, TRUE);
+						ad->auth_string = NULL;
+					}
+					for (plist = set ? set->holders : NULL;
+					     plist; plist = plist->next) {
+						GdaHolder *holder = GDA_HOLDER (plist->data);
+						const GValue *cvalue = NULL;
+						if (gda_holder_is_valid (holder))
+							cvalue = gda_holder_get_value (holder);
+						if (cvalue) {
+							gchar *r1, *r2;
+							r1 = gda_value_stringify (cvalue);
+							r2 = gda_rfc1738_encode (r1);
+							g_free (r1);
+							if (r2) {
+								r1 = gda_rfc1738_encode (gda_holder_get_id (holder));
+								if (ad->auth_string)
+									g_string_append_c (ad->auth_string, ';');
+								else
+									ad->auth_string = g_string_new ("");
+								g_string_append (ad->auth_string, r1);
+								g_string_append_c (ad->auth_string, '=');
+								g_string_append (ad->auth_string, r2);
+							
+								g_free (r1);
+								g_free (r2);
+							}
+						}
+					}
 					gtk_widget_set_sensitive (ad->auth_widget, FALSE);
 					ad->jobid = gda_thread_wrapper_execute (ad->wrapper,
 										(GdaThreadWrapperFunc) sub_thread_open_cnc,
