@@ -101,6 +101,7 @@ push_wrapper_job (BrowserConnection *bcnc, guint job_id, JobType job_type, const
 	wj->job_type = job_type;
 	if (reason)
 		wj->reason = g_strdup (reason);
+
 	bcnc->priv->wrapper_jobs = g_slist_append (bcnc->priv->wrapper_jobs, wj);
 
 	if (! bcnc->priv->wrapper_jobs->next)
@@ -254,6 +255,8 @@ browser_connection_init (BrowserConnection *bcnc)
 	bcnc->priv->bfav = NULL;
 
 	bcnc->priv->store_cnc = NULL;
+
+	bcnc->priv->variables = NULL;
 }
 
 static void
@@ -424,6 +427,9 @@ browser_connection_dispose (GObject *object)
 
 	bcnc = BROWSER_CONNECTION (object);
 	if (bcnc->priv) {
+		if (bcnc->priv->variables)
+			g_object_unref (bcnc->priv->variables);
+
 		if (bcnc->priv->store_cnc)
 			g_object_unref (bcnc->priv->store_cnc);
 
@@ -458,10 +464,6 @@ browser_connection_dispose (GObject *object)
 		}
 		if (bcnc->priv->parser)
 			g_object_unref (bcnc->priv->parser);
-		if (bcnc->priv->variables) {
-			g_slist_foreach (bcnc->priv->variables, (GFunc) g_object_unref, NULL);
-			g_slist_free (bcnc->priv->variables);
-		}
 		if (bcnc->priv->bfav) {
 			g_signal_handlers_disconnect_by_func (bcnc->priv->bfav,
 							      G_CALLBACK (fav_changed_cb), bcnc);
@@ -713,9 +715,10 @@ browser_connection_update_meta_data (BrowserConnection *bcnc)
 	if (bcnc->priv->wrapper_jobs) {
 		WrapperJob *wj;
 		wj = (WrapperJob*) g_slist_last (bcnc->priv->wrapper_jobs)->data;
-		if (wj->job_type == JOB_TYPE_META_STORE_UPDATE)
+		if (wj->job_type == JOB_TYPE_META_STORE_UPDATE) {
 			/* nothing to do */
 			return;
+		}
 	}
 
 	guint job_id;
@@ -985,6 +988,50 @@ browser_connection_execute_statement (BrowserConnection *bcnc,
 	return job_id;
 }
 
+typedef struct {
+	GdaConnection *cnc;
+	GdaDataModel *model;
+} RerunSelectData;
+
+/* executed in @bcnc->priv->wrapper's sub thread */
+static gpointer
+wrapper_rerun_select (RerunSelectData *data, GError **error)
+{
+	gboolean retval;
+
+	retval = gda_data_select_rerun (GDA_DATA_SELECT (data->model), error);
+	return retval ? data->model : (gpointer) 0x01;
+}
+
+/**
+ * browser_connection_rerun_select
+ */
+guint
+browser_connection_rerun_select (BrowserConnection *bcnc,
+				 GdaDataModel *model,
+				 GError **error)
+{
+	g_return_val_if_fail (BROWSER_IS_CONNECTION (bcnc), 0);
+	g_return_val_if_fail (GDA_IS_DATA_SELECT (model), 0);
+
+	RerunSelectData *data;
+	guint job_id;
+
+	data = g_new0 (RerunSelectData, 1);
+	data->cnc = bcnc->priv->cnc;
+	data->model = model;
+
+	job_id = gda_thread_wrapper_execute (bcnc->priv->wrapper,
+					     (GdaThreadWrapperFunc) wrapper_rerun_select,
+					     data, (GDestroyNotify) g_free, error);
+	if (job_id > 0)
+		push_wrapper_job (bcnc, job_id, JOB_TYPE_STATEMENT_EXECUTE,
+				  _("Executing a query"));
+
+	return job_id;
+}
+
+
 /**
  * browser_connection_execution_get_result
  * @bcnc: a #BrowserConnection
@@ -1353,3 +1400,94 @@ browser_connection_get_table_column_attribute  (BrowserConnection *bcnc,
 
 	return retval;
 }
+
+/**
+ * browser_connection_keep_variables
+ * @bcnc: a #BrowserConnection object
+ * @set: a #GdaSet containing variables for which a copy has to be done
+ *
+ * Makes a copy of the variables in @set and keep them in @bcnc. Retreive them
+ * using browser_connection_load_variables()
+ */
+void
+browser_connection_keep_variables (BrowserConnection *bcnc, GdaSet *set)
+{
+	g_return_if_fail (BROWSER_IS_CONNECTION (bcnc));
+	if (!set)
+		return;
+	g_return_if_fail (GDA_IS_SET (set));
+
+	if (! bcnc->priv->variables) {
+		bcnc->priv->variables = gda_set_copy (set);
+		return;
+	}
+
+	GSList *list;
+	for (list = set->holders; list; list = list->next) {
+		GdaHolder *nh, *eh;
+		nh = GDA_HOLDER (list->data);
+		eh = gda_set_get_holder (bcnc->priv->variables, gda_holder_get_id (nh));
+		if (eh) {
+			if (gda_holder_get_g_type (nh) == gda_holder_get_g_type (eh)) {
+				const GValue *cvalue;
+				cvalue = gda_holder_get_value (nh);
+				gda_holder_set_value (eh, cvalue, NULL);
+			}
+			else {
+				gda_set_remove_holder (bcnc->priv->variables, eh);
+				eh = gda_holder_copy (nh);
+				gda_set_add_holder (bcnc->priv->variables, eh);
+				g_object_unref (eh);
+			}
+		}
+		else {
+			eh = gda_holder_copy (nh);
+			gda_set_add_holder (bcnc->priv->variables, eh);
+			g_object_unref (eh);
+		}
+	}
+}
+
+/**
+ * browser_connection_load_variables
+ * @bcnc: a #BrowserConnection object
+ * @set: a #GdaSet which will in the end contain (if any) variables stored in @bcnc
+ *
+ * For each #GdaHolder in @set, set the value if one is available in @bcnc.
+ */
+void
+browser_connection_load_variables (BrowserConnection *bcnc, GdaSet *set)
+{
+	g_return_if_fail (BROWSER_IS_CONNECTION (bcnc));
+	if (!set)
+		return;
+	g_return_if_fail (GDA_IS_SET (set));
+
+	if (! bcnc->priv->variables)
+		return;
+
+	GSList *list;
+	for (list = set->holders; list; list = list->next) {
+		GdaHolder *nh, *eh;
+		nh = GDA_HOLDER (list->data);
+		eh = gda_set_get_holder (bcnc->priv->variables, gda_holder_get_id (nh));
+		if (eh) {
+			if (gda_holder_get_g_type (nh) == gda_holder_get_g_type (eh)) {
+				const GValue *cvalue;
+				cvalue = gda_holder_get_value (eh);
+				gda_holder_set_value (nh, cvalue, NULL);
+			}
+			else if (g_value_type_transformable (gda_holder_get_g_type (eh),
+							     gda_holder_get_g_type (nh))) {
+				const GValue *evalue;
+				GValue *nvalue;
+				evalue = gda_holder_get_value (eh);
+				nvalue = gda_value_new (gda_holder_get_g_type (nh));
+				if (g_value_transform (evalue, nvalue))
+					gda_holder_take_value (nh, nvalue, NULL);
+				else
+					gda_value_free (nvalue);
+			}
+		}
+	}	
+}
