@@ -86,6 +86,9 @@ guint               browser_connection_execute_statement      (BrowserConnection
 							       GdaStatementModelUsage model_usage,
 							       gboolean need_last_insert_row,
 							       GError **error);
+guint               browser_connection_rerun_select           (BrowserConnection *bcnc,
+							       GdaDataModel *model,
+							       GError **error);
 GObject            *browser_connection_execution_get_result   (BrowserConnection *bcnc,
 							       guint exec_id,
 							       GdaSet **last_insert_row, GError **error);
@@ -115,6 +118,13 @@ gchar               *browser_connection_get_table_column_attribute (BrowserConne
 								    const gchar *attr_name,
 								    GError **error);
 
+/*
+ * Variables used at various places and for which a copy of the last recent value
+ * is stored in the BrowserConnection object
+ */
+void                 browser_connection_keep_variables (BrowserConnection *bcnc, GdaSet *set);
+void                 browser_connection_load_variables (BrowserConnection *bcnc, GdaSet *set);
+
 G_END_DECLS
 
 #endif
