@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 The GNOME Foundation
+ * Copyright (C) 2009 - 2010 The GNOME Foundation
  *
  * AUTHORS:
  *      Vivien Malerba <malerba@gnome-db.org>
@@ -22,7 +22,6 @@
 
 #include <glib/gi18n-lib.h>
 #include <string.h>
-#include <gtk/gtk.h>
 #include "query-console.h"
 #include "../dnd.h"
 #include "../support.h"
@@ -37,17 +36,6 @@
 #include <libgda/sql-parser/gda-sql-parser.h>
 #include <libgda-ui/libgda-ui.h>
 
-#define VARIABLES_HELP _("<small>This area allows to give values to\n" \
-			 "variables defined in the SQL code\n"		\
-			 "using the following syntax:\n"		\
-			 "<b><tt>##&lt;variable name&gt;::&lt;type&gt;[::null]</tt></b>\n" \
-			 "For example:\n"				\
-			 "<span foreground=\"#4e9a06\"><b><tt>##id::int</tt></b></span>\n      defines <b>id</b> as a non NULL integer\n" \
-			 "<span foreground=\"#4e9a06\"><b><tt>##age::string::null</tt></b></span>\n      defines <b>age</b> as a a string\n\n" \
-			 "Valid types are: <tt>string</tt>, <tt>boolean</tt>, <tt>int</tt>,\n" \
-			 "<tt>date</tt>, <tt>time</tt>, <tt>timestamp</tt>, <tt>guint</tt>, <tt>blob</tt> and\n" \
-			 "<tt>binary</tt></small>")
-
 /*
  * Statement execution structures
  */
@@ -109,12 +97,15 @@ struct _QueryConsolePrivate {
 	BrowserConnection *bcnc;
 	GdaSqlParser *parser;
 
+	GtkActionGroup *agroup;
+
 	CcGrayBar *header;
 	GtkWidget *vpaned; /* top=>query editor, bottom=>results */
 
 	QueryEditor *editor;
+	GtkWidget   *exec_button;
+	GtkWidget   *indent_button;
 	guint params_compute_id; /* timout ID to compute params */
-	GdaSet *past_params; /* keeps values given to old params */
 	GdaSet *params; /* execution params */
 	GtkWidget *params_popup; /* popup shown when invalid params are required */
 
@@ -146,6 +137,7 @@ static const gchar         *query_console_page_get_actions_ui (BrowserPage *page
 static GtkWidget           *query_console_page_get_tab_label (BrowserPage *page, GtkWidget **out_close_button);
 
 enum {
+	DUMMY,
 	LAST_SIGNAL
 };
 
@@ -194,11 +186,12 @@ query_console_init (QueryConsole *tconsole, QueryConsoleClass *klass)
 	tconsole->priv = g_new0 (QueryConsolePrivate, 1);
 	tconsole->priv->parser = NULL;
 	tconsole->priv->params_compute_id = 0;
-	tconsole->priv->past_params = NULL;
 	tconsole->priv->params = NULL;
 	tconsole->priv->params_popup = NULL;
+	tconsole->priv->agroup = NULL;
 }
-
+static void connection_busy_cb (BrowserConnection *bcnc, gboolean is_busy,
+				gchar *reason, QueryConsole *tconsole);
 static void
 query_console_dispose (GObject *object)
 {
@@ -210,18 +203,21 @@ query_console_dispose (GObject *object)
 			g_source_remove (tconsole->priv->current_exec_id);
 		if (tconsole->priv->current_exec)
 			execution_batch_free (tconsole->priv->current_exec);
-		if (tconsole->priv->bcnc)
+		if (tconsole->priv->bcnc) {
+			g_signal_handlers_disconnect_by_func (tconsole->priv->bcnc,
+							      G_CALLBACK (connection_busy_cb), tconsole);
 			g_object_unref (tconsole->priv->bcnc);
+		}
 		if (tconsole->priv->parser)
 			g_object_unref (tconsole->priv->parser);
-		if (tconsole->priv->past_params)
-			g_object_unref (tconsole->priv->past_params);
 		if (tconsole->priv->params)
 			g_object_unref (tconsole->priv->params);
 		if (tconsole->priv->params_compute_id)
 			g_source_remove (tconsole->priv->params_compute_id);
 		if (tconsole->priv->params_popup)
 			gtk_widget_destroy (tconsole->priv->params_popup);
+		if (tconsole->priv->agroup)
+			g_object_unref (tconsole->priv->agroup);
 
 		g_free (tconsole->priv);
 		tconsole->priv = NULL;
@@ -260,9 +256,6 @@ query_console_get_type (void)
 	return type;
 }
 
-static GtkWidget *make_small_button (gboolean is_toggle,
-				     const gchar *label, const gchar *stock_id, const gchar *tooltip);
-
 static void editor_changed_cb (QueryEditor *editor, QueryConsole *tconsole);
 static void editor_execute_request_cb (QueryEditor *editor, QueryConsole *tconsole);
 static void sql_clear_clicked_cb (GtkButton *button, QueryConsole *tconsole);
@@ -359,6 +352,7 @@ query_console_new (BrowserConnection *bcnc)
 
 	wid = gtk_label_new ("");
 	gtk_label_set_markup (GTK_LABEL (wid), VARIABLES_HELP);
+	gtk_misc_set_alignment (GTK_MISC (wid), -1, 0.);
 	gtk_container_add (GTK_CONTAINER (tconsole->priv->params_form_box), wid);
 	tconsole->priv->params_form = wid;
 	
@@ -366,30 +360,32 @@ query_console_new (BrowserConnection *bcnc)
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_END);
 	gtk_box_pack_start (GTK_BOX (hbox), bbox, FALSE, FALSE, 5);
 
-	button = make_small_button (FALSE, _("Clear"), GTK_STOCK_CLEAR, _("Clear the editor"));
+	button = browser_make_small_button (FALSE, _("Clear"), GTK_STOCK_CLEAR, _("Clear the editor's\ncontents"));
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (sql_clear_clicked_cb), tconsole);
 
-	button = make_small_button (TRUE, _("Variables"), NULL, _("Show variables needed\nto execute SQL"));
+	button = browser_make_small_button (TRUE, _("Variables"), NULL, _("Show variables needed\nto execute SQL"));
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	tconsole->priv->params_toggle = GTK_TOGGLE_BUTTON (button);
 	g_signal_connect (button, "toggled",
 			  G_CALLBACK (sql_variables_clicked_cb), tconsole);
 
-	button = make_small_button (FALSE, _("Execute"), GTK_STOCK_EXECUTE, _("Execute SQL in editor"));
+	button = browser_make_small_button (FALSE, _("Execute"), GTK_STOCK_EXECUTE, _("Execute SQL in editor"));
+	tconsole->priv->exec_button = button;
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (sql_execute_clicked_cb), tconsole);
 	
-	button = make_small_button (FALSE, _("Indent"), GTK_STOCK_INDENT, _("Indent SQL in editor\n"
+	button = browser_make_small_button (FALSE, _("Indent"), GTK_STOCK_INDENT, _("Indent SQL in editor\n"
 									    "and make the code more readable\n"
 									    "(removes comments)"));
+	tconsole->priv->indent_button = button;
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (sql_indent_clicked_cb), tconsole);
 
-	button = make_small_button (FALSE, _("Favorite"), STOCK_ADD_BOOKMARK, _("Add SQL to favorite"));
+	button = browser_make_small_button (FALSE, _("Favorite"), STOCK_ADD_BOOKMARK, _("Add SQL to favorite"));
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (sql_favorite_clicked_cb), tconsole);
@@ -421,14 +417,14 @@ query_console_new (BrowserConnection *bcnc)
 	gtk_box_pack_start (GTK_BOX (vbox), bbox, FALSE, FALSE, 0);
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_END);
 
-	button = make_small_button (FALSE, _("Copy"), GTK_STOCK_COPY, _("Copy selected history\nto editor"));
+	button = browser_make_small_button (FALSE, _("Copy"), GTK_STOCK_COPY, _("Copy selected history\nto editor"));
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (history_copy_clicked_cb), tconsole);
 	tconsole->priv->history_copy_button = button;
 	gtk_widget_set_sensitive (button, FALSE);
 
-	button = make_small_button (FALSE, _("Clear"), GTK_STOCK_CLEAR, _("Clear history"));
+	button = browser_make_small_button (FALSE, _("Clear"), GTK_STOCK_CLEAR, _("Clear history"));
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (history_clear_clicked_cb), tconsole);
@@ -454,46 +450,29 @@ query_console_new (BrowserConnection *bcnc)
         gtk_widget_show_all (vpaned);
 	gtk_widget_hide (tconsole->priv->params_top);
 
+	/* busy connection handling */
+	gchar *reason = NULL;
+	if (browser_connection_is_busy (tconsole->priv->bcnc, &reason)) {
+		connection_busy_cb (tconsole->priv->bcnc, TRUE, reason, tconsole);
+		g_free (reason);
+	}
+	g_signal_connect (tconsole->priv->bcnc, "busy",
+			  G_CALLBACK (connection_busy_cb), tconsole);
+
 	return (GtkWidget*) tconsole;
 }
 
-static GtkWidget *
-make_small_button (gboolean is_toggle, const gchar *label, const gchar *stock_id, const gchar *tooltip)
+static void
+connection_busy_cb (BrowserConnection *bcnc, gboolean is_busy, gchar *reason, QueryConsole *tconsole)
 {
-	GtkWidget *button, *hbox = NULL;
+	gtk_widget_set_sensitive (tconsole->priv->exec_button, !is_busy);
+	gtk_widget_set_sensitive (tconsole->priv->indent_button, !is_busy);
 
-	if (is_toggle)
-		button = gtk_toggle_button_new ();
-	else
-		button = gtk_button_new ();
-	if (label && stock_id) {
-		hbox = gtk_hbox_new (FALSE, 0);
-		gtk_container_add (GTK_CONTAINER (button), hbox);
-		gtk_widget_show (hbox);
+	if (tconsole->priv->agroup) {
+		GtkAction *action;
+		action = gtk_action_group_get_action (tconsole->priv->agroup, "ExecuteQuery");
+		gtk_action_set_sensitive (action, !is_busy);
 	}
-
-	if (stock_id) {
-		GtkWidget *image;
-		image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_MENU);
-		if (hbox)
-			gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
-		else
-			gtk_container_add (GTK_CONTAINER (button), image);
-		gtk_widget_show (image);
-	}
-	if (label) {
-		GtkWidget *wid;
-		wid = gtk_label_new (label);
-		if (hbox)
-			gtk_box_pack_start (GTK_BOX (hbox), wid, FALSE, FALSE, 5);
-		else
-			gtk_container_add (GTK_CONTAINER (button), wid);
-		gtk_widget_show (wid);
-	}
-
-	if (tooltip)
-		gtk_widget_set_tooltip_text (button, tooltip);
-	return button;
 }
 
 static void
@@ -516,8 +495,8 @@ history_changed_cb (QueryEditor *history, QueryConsole *tconsole)
 	else
 		query_result_show_history_batch (QUERY_RESULT (tconsole->priv->query_result), NULL);
 
-	gtk_widget_set_sensitive (tconsole->priv->history_del_button, act);
 	gtk_widget_set_sensitive (tconsole->priv->history_copy_button, act);
+	gtk_widget_set_sensitive (tconsole->priv->history_del_button, ! query_editor_history_is_empty (qe));
 }
 
 static void
@@ -562,16 +541,11 @@ compute_params (QueryConsole *tconsole)
 	GdaBatch *batch;
 
 	if (tconsole->priv->params) {
-		if (tconsole->priv->past_params) {
-			gda_set_merge_with_set (tconsole->priv->params, tconsole->priv->past_params);
-			g_object_unref (tconsole->priv->past_params);
-			tconsole->priv->past_params = tconsole->priv->params;
-		}
-		else
-			tconsole->priv->past_params = tconsole->priv->params;
+		browser_connection_keep_variables (tconsole->priv->bcnc, tconsole->priv->params);
+		g_object_unref (tconsole->priv->params);
 	}
-
 	tconsole->priv->params = NULL;
+
 	if (tconsole->priv->params_form) {
 		gtk_widget_destroy (tconsole->priv->params_form);
 		tconsole->priv->params_form = NULL;		
@@ -591,8 +565,8 @@ compute_params (QueryConsole *tconsole)
 			if (tconsole->priv->params) {
 				show_variables = TRUE;
 				tconsole->priv->params_form = gdaui_basic_form_new (tconsole->priv->params);
-				gdaui_basic_form_show_entry_actions (GDAUI_BASIC_FORM (tconsole->priv->params_form),
-								     TRUE);
+				g_object_set ((GObject*) tconsole->priv->params_form,
+					      "show-actions", TRUE, NULL);
 			}
 			else {
 				tconsole->priv->params_form = gtk_label_new ("");
@@ -608,35 +582,7 @@ compute_params (QueryConsole *tconsole)
 		gtk_widget_show (tconsole->priv->params_form);
 		g_object_unref (batch);
 
-		if (tconsole->priv->past_params && tconsole->priv->params) {
-			/* copy the values from tconsole->priv->past_params to tconsole->priv->params */
-			GSList *list;
-			for (list = tconsole->priv->params->holders; list; list = list->next) {
-				GdaHolder *oldh, *newh;
-				newh = GDA_HOLDER (list->data);
-				oldh = gda_set_get_holder (tconsole->priv->past_params, gda_holder_get_id  (newh));
-				if (oldh) {
-					GType otype, ntype;
-					otype = gda_holder_get_g_type (oldh);
-					ntype = gda_holder_get_g_type (newh);
-					if (otype == ntype) {
-						const GValue *ovalue;
-						ovalue = gda_holder_get_value (oldh);
-						gda_holder_set_value (newh, ovalue, NULL);
-					}
-					else if (g_value_type_transformable (otype, ntype)) {
-						const GValue *ovalue;
-						GValue *nvalue;
-						ovalue = gda_holder_get_value (oldh);
-						nvalue = gda_value_new (ntype);
-						if (g_value_transform (ovalue, nvalue))
-							gda_holder_take_value (newh, nvalue, NULL);
-						else
-							gda_value_free  (nvalue);
-					}
-				}
-			}
-		}
+		browser_connection_load_variables (tconsole->priv->bcnc, tconsole->priv->params);
 		if (tconsole->priv->params && show_variables &&
 		    gda_set_is_valid (tconsole->priv->params, NULL))
 			show_variables = FALSE;
@@ -666,7 +612,10 @@ editor_changed_cb (QueryEditor *editor, QueryConsole *tconsole)
 static void
 editor_execute_request_cb (QueryEditor *editor, QueryConsole *tconsole)
 {
-	sql_execute_clicked_cb (NULL, tconsole);
+	gboolean sensitive;
+	g_object_get (tconsole->priv->exec_button, "sensitive", &sensitive, NULL);
+	if (sensitive)
+		sql_execute_clicked_cb (NULL, tconsole);
 }
 	
 static void
@@ -682,6 +631,7 @@ static void
 sql_clear_clicked_cb (GtkButton *button, QueryConsole *tconsole)
 {
 	query_editor_set_text (tconsole->priv->editor, NULL);
+	gtk_widget_grab_focus (GTK_WIDGET (tconsole->priv->editor));
 }
 
 static void
@@ -754,10 +704,18 @@ popup_container_position_func (PopupContainer *cont, gint *out_x, gint *out_y)
 	console = g_object_get_data (G_OBJECT (cont), "console");
 	top = gtk_widget_get_toplevel (console);	
         gtk_widget_size_request ((GtkWidget*) cont, &req);
+#if GTK_CHECK_VERSION(2,18,0)
+	GtkAllocation alloc;
+        gdk_window_get_origin (gtk_widget_get_window (top), &x, &y);
+	gtk_widget_get_allocation (top, &alloc);
+	x += (alloc.width - req.width) / 2;
+	y += (alloc.height - req.height) / 2;
+#else
         gdk_window_get_origin (top->window, &x, &y);
 	
 	x += (top->allocation.width - req.width) / 2;
 	y += (top->allocation.height - req.height) / 2;
+#endif
 
         if (x < 0)
                 x = 0;
@@ -847,9 +805,9 @@ sql_execute_clicked_cb (GtkButton *button, QueryConsole *tconsole)
 			}
 
 			cont = g_object_get_data (G_OBJECT (tconsole->priv->params_popup), "cont");
-			form = gdaui_basic_form_new (tconsole->priv->params);			
-			gdaui_basic_form_show_entry_actions (GDAUI_BASIC_FORM (form), TRUE);
-			g_signal_connect (form, "param-changed",
+			form = gdaui_basic_form_new (tconsole->priv->params);
+			g_object_set ((GObject*) form, "show-actions", TRUE, NULL);
+			g_signal_connect (form, "holder-changed",
 					  G_CALLBACK (params_form_changed_cb), tconsole);
 
 			gtk_box_pack_start (GTK_BOX (cont), form, TRUE, TRUE, 0);
@@ -894,7 +852,6 @@ sql_execute_clicked_cb (GtkButton *button, QueryConsole *tconsole)
 	ebatch->batch = batch;
 	g_get_current_time (&(ebatch->start_time));
 	ebatch->hist_batch = query_editor_history_batch_new (ebatch->start_time, tconsole->priv->params);
-	query_editor_start_history_batch (tconsole->priv->history, ebatch->hist_batch);
 
 	stmt_list = gda_batch_get_statements (batch);
 	for (list = stmt_list; list; list = list->next) {
@@ -944,6 +901,10 @@ query_exec_fetch_cb (QueryConsole *tconsole)
 										 estmt->exec_id, NULL,
 										 &(estmt->exec_error));
 			if (estmt->result || estmt->exec_error) {
+				ExecutionBatch *ebatch;
+				ebatch = tconsole->priv->current_exec;
+				query_editor_start_history_batch (tconsole->priv->history, ebatch->hist_batch);
+
 				QueryEditorHistoryItem *history;
 				GdaSqlStatement *sqlst;
 				g_object_get (G_OBJECT (estmt->stmt), "structure", &sqlst, NULL);
@@ -1092,11 +1053,18 @@ static const gchar *ui_actions_console =
 static GtkActionGroup *
 query_console_page_get_actions_group (BrowserPage *page)
 {
-	GtkActionGroup *agroup;
-	agroup = gtk_action_group_new ("QueryExecConsoleActions");
-	gtk_action_group_add_actions (agroup, ui_actions, G_N_ELEMENTS (ui_actions), page);
-	
-	return agroup;
+	QueryConsole *tconsole;
+	tconsole = QUERY_CONSOLE (page);
+	if (! tconsole->priv->agroup) {
+		tconsole->priv->agroup = gtk_action_group_new ("QueryExecConsoleActions");
+		gtk_action_group_add_actions (tconsole->priv->agroup,
+					      ui_actions, G_N_ELEMENTS (ui_actions), page);
+
+		GtkAction *action;
+		action = gtk_action_group_get_action (tconsole->priv->agroup, "ExecuteQuery");
+		gtk_action_set_sensitive (action, !browser_connection_is_busy (tconsole->priv->bcnc, NULL));
+	}
+	return g_object_ref (tconsole->priv->agroup);
 }
 
 static const gchar *
